@@ -21,26 +21,20 @@
 
 /**
  * SECTION:element-flacdec
- * @seealso: flacenc
+ * @see_also: #GstFlacEnc
  *
- * <refsect2>
- * <para>
  * flacdec decodes FLAC streams.
  * <ulink url="http://flac.sourceforge.net/">FLAC</ulink>
  * is a Free Lossless Audio Codec.
- * </para>
+ *
+ * <refsect2>
  * <title>Example launch line</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch filesrc location=media/small/dark.441-16-s.flac ! flacdec ! audioconvert ! audioresample ! autoaudiosink
- * </programlisting>
- * </para>
- * <title>Another example launch line</title>
- * <para>
- * <programlisting>
+ * ]|
+ * |[
  * gst-launch gnomevfssrc location=http://gstreamer.freedesktop.org/media/small/dark.441-16-s.flac ! flacdec ! audioconvert ! audioresample ! queue min-threshold-buffers=10 ! autoaudiosink
- * </programlisting>
- * </para>
+ * ]|
  * </refsect2>
  */
 
@@ -54,44 +48,86 @@
 #include "gstflacdec.h"
 #include <gst/gst-i18n-plugin.h>
 #include <gst/gsttagsetter.h>
-
+#include <gst/base/gsttypefindhelper.h>
+#include <gst/audio/multichannel.h>
 #include <gst/tag/tag.h>
 
+/* Taken from http://flac.sourceforge.net/format.html#frame_header */
+static const GstAudioChannelPosition channel_positions[8][8] = {
+  {GST_AUDIO_CHANNEL_POSITION_FRONT_MONO},
+  {GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+      GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT}, {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+      GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER}, {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
+      GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT}, {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
+      GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT}, {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_LFE,
+        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
+      GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT},
+  /* FIXME: 7/8 channel layouts are not defined in the FLAC specs */
+  {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_LFE,
+      GST_AUDIO_CHANNEL_POSITION_REAR_CENTER}, {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_LFE,
+        GST_AUDIO_CHANNEL_POSITION_SIDE_LEFT,
+      GST_AUDIO_CHANNEL_POSITION_SIDE_RIGHT}
+};
+
 GST_DEBUG_CATEGORY_STATIC (flacdec_debug);
 #define GST_CAT_DEFAULT flacdec_debug
 
-static GstPadTemplate *src_template, *sink_template;
-
-static const GstElementDetails flacdec_details =
-GST_ELEMENT_DETAILS ("FLAC audio decoder",
-    "Codec/Decoder/Audio",
-    "Decodes FLAC lossless audio streams",
-    "Wim Taymans <wim@fluendo.com>");
-
-
 static void gst_flac_dec_finalize (GObject * object);
-
 static void gst_flac_dec_loop (GstPad * pad);
+
 static GstStateChangeReturn gst_flac_dec_change_state (GstElement * element,
     GstStateChange transition);
 static const GstQueryType *gst_flac_dec_get_src_query_types (GstPad * pad);
+
 static const GstQueryType *gst_flac_dec_get_sink_query_types (GstPad * pad);
+
 static gboolean gst_flac_dec_sink_query (GstPad * pad, GstQuery * query);
+
 static gboolean gst_flac_dec_src_query (GstPad * pad, GstQuery * query);
+
 static gboolean gst_flac_dec_convert_src (GstPad * pad, GstFormat src_format,
     gint64 src_value, GstFormat * dest_format, gint64 * dest_value);
 static gboolean gst_flac_dec_src_event (GstPad * pad, GstEvent * event);
+
 static gboolean gst_flac_dec_sink_activate (GstPad * sinkpad);
+
 static gboolean gst_flac_dec_sink_activate_pull (GstPad * sinkpad,
     gboolean active);
 static gboolean gst_flac_dec_sink_activate_push (GstPad * sinkpad,
     gboolean active);
-static void gst_flac_dec_send_newsegment (GstFlacDec * flacdec,
-    gboolean update);
 static gboolean gst_flac_dec_sink_event (GstPad * pad, GstEvent * event);
+
 static GstFlowReturn gst_flac_dec_chain (GstPad * pad, GstBuffer * buf);
+
 static void gst_flac_dec_reset_decoders (GstFlacDec * flacdec);
+
 static void gst_flac_dec_setup_seekable_decoder (GstFlacDec * flacdec);
+
 static void gst_flac_dec_setup_stream_decoder (GstFlacDec * flacdec);
 
 #ifdef LEGACY_FLAC
@@ -161,30 +197,49 @@ static void gst_flac_dec_error_callback_stream (const FLAC__StreamDecoder *
 #endif
 
 GST_BOILERPLATE (GstFlacDec, gst_flac_dec, GstElement, GST_TYPE_ELEMENT);
+
+/* FIXME 0.11: Use width=32 for all depths and let audioconvert
+ * handle the conversions instead of doing it ourself.
+ */
+static const GstElementDetails vorbis_dec_details =
+GST_ELEMENT_DETAILS ("Vorbis audio decoder",
+    "Codec/Decoder/Audio",
+    "decode raw vorbis streams to float audio",
+    "Benjamin Otte <in7y118@public.uni-hamburg.de>");
+
 #define GST_FLAC_DEC_SRC_CAPS                             \
     "audio/x-raw-int, "                                   \
     "endianness = (int) BYTE_ORDER, "                     \
     "signed = (boolean) true, "                           \
     "width = (int) { 8, 16, 32 }, "                       \
-    "depth = (int) { 8, 12, 16, 20, 24, 32 }, "           \
-    "rate = (int) [ 8000, 96000 ], "                      \
+    "depth = (int) [ 4, 32 ], "                           \
+    "rate = (int) [ 1, 655350 ], "                        \
     "channels = (int) [ 1, 8 ]"
+
+static GstStaticPadTemplate flac_dec_src_factory =
+GST_STATIC_PAD_TEMPLATE ("src",
+    GST_PAD_SRC,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS (GST_FLAC_DEC_SRC_CAPS));
+static GstStaticPadTemplate flac_dec_sink_factory =
+GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS ("audio/x-flac")
+    );
+
 static void
 gst_flac_dec_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-  GstCaps *raw_caps, *flac_caps;
 
-  raw_caps = gst_caps_from_string (GST_FLAC_DEC_SRC_CAPS);
-  flac_caps = gst_caps_new_simple ("audio/x-flac", NULL);
-
-  sink_template = gst_pad_template_new ("sink", GST_PAD_SINK,
-      GST_PAD_ALWAYS, flac_caps);
-  src_template = gst_pad_template_new ("src", GST_PAD_SRC,
-      GST_PAD_ALWAYS, raw_caps);
-  gst_element_class_add_pad_template (element_class, sink_template);
-  gst_element_class_add_pad_template (element_class, src_template);
-  gst_element_class_set_details (element_class, &flacdec_details);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&flac_dec_src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&flac_dec_sink_factory));
+  gst_element_class_set_details_simple (element_class, "FLAC audio decoder",
+      "Codec/Decoder/Audio",
+      "Decodes FLAC lossless audio streams", "Wim Taymans <wim@fluendo.com>");
 
   GST_DEBUG_CATEGORY_INIT (flacdec_debug, "flacdec", 0, "flac decoder");
 }
@@ -193,6 +248,7 @@ static void
 gst_flac_dec_class_init (GstFlacDecClass * klass)
 {
   GstElementClass *gstelement_class;
+
   GObjectClass *gobject_class;
 
   gstelement_class = (GstElementClass *) klass;
@@ -207,7 +263,8 @@ gst_flac_dec_class_init (GstFlacDecClass * klass)
 static void
 gst_flac_dec_init (GstFlacDec * flacdec, GstFlacDecClass * klass)
 {
-  flacdec->sinkpad = gst_pad_new_from_template (sink_template, "sink");
+  flacdec->sinkpad =
+      gst_pad_new_from_static_template (&flac_dec_sink_factory, "sink");
   gst_pad_set_activate_function (flacdec->sinkpad,
       GST_DEBUG_FUNCPTR (gst_flac_dec_sink_activate));
   gst_pad_set_activatepull_function (flacdec->sinkpad,
@@ -224,7 +281,8 @@ gst_flac_dec_init (GstFlacDec * flacdec, GstFlacDecClass * klass)
       GST_DEBUG_FUNCPTR (gst_flac_dec_chain));
   gst_element_add_pad (GST_ELEMENT (flacdec), flacdec->sinkpad);
 
-  flacdec->srcpad = gst_pad_new_from_template (src_template, "src");
+  flacdec->srcpad =
+      gst_pad_new_from_static_template (&flac_dec_src_factory, "src");
   gst_pad_set_query_type_function (flacdec->srcpad,
       GST_DEBUG_FUNCPTR (gst_flac_dec_get_src_query_types));
   gst_pad_set_query_function (flacdec->srcpad,
@@ -261,6 +319,19 @@ gst_flac_dec_reset_decoders (GstFlacDec * flacdec)
     flacdec->adapter = NULL;
   }
 
+  if (flacdec->close_segment) {
+    gst_event_unref (flacdec->close_segment);
+    flacdec->close_segment = NULL;
+  }
+  if (flacdec->start_segment) {
+    gst_event_unref (flacdec->start_segment);
+    flacdec->start_segment = NULL;
+  }
+  if (flacdec->tags) {
+    gst_tag_list_free (flacdec->tags);
+    flacdec->tags = NULL;
+  }
+
   flacdec->segment.last_stop = 0;
   flacdec->offset = 0;
   flacdec->init = TRUE;
@@ -271,6 +342,10 @@ gst_flac_dec_setup_seekable_decoder (GstFlacDec * dec)
 {
   gst_flac_dec_reset_decoders (dec);
 
+  dec->tags = gst_tag_list_new ();
+  gst_tag_list_add (dec->tags, GST_TAG_MERGE_REPLACE,
+      GST_TAG_AUDIO_CODEC, "FLAC", NULL);
+
 #ifdef LEGACY_FLAC
   dec->seekable_decoder = FLAC__seekable_stream_decoder_new ();
 
@@ -297,6 +372,8 @@ gst_flac_dec_setup_seekable_decoder (GstFlacDec * dec)
   dec->seekable_decoder = FLAC__stream_decoder_new ();
   FLAC__stream_decoder_set_metadata_respond (dec->seekable_decoder,
       FLAC__METADATA_TYPE_VORBIS_COMMENT);
+  FLAC__stream_decoder_set_metadata_respond (dec->seekable_decoder,
+      FLAC__METADATA_TYPE_PICTURE);
   FLAC__stream_decoder_set_md5_checking (dec->seekable_decoder, false); /* no point calculating since it's never checked here */
 #endif
 }
@@ -306,6 +383,10 @@ gst_flac_dec_setup_stream_decoder (GstFlacDec * dec)
 {
   gst_flac_dec_reset_decoders (dec);
 
+  dec->tags = gst_tag_list_new ();
+  gst_tag_list_add (dec->tags, GST_TAG_MERGE_REPLACE,
+      GST_TAG_AUDIO_CODEC, "FLAC", NULL);
+
   dec->adapter = gst_adapter_new ();
 
   dec->stream_decoder = FLAC__stream_decoder_new ();
@@ -326,6 +407,8 @@ gst_flac_dec_setup_stream_decoder (GstFlacDec * dec)
   FLAC__stream_decoder_set_md5_checking (dec->stream_decoder, false);   /* no point calculating since it's never checked here */
   FLAC__stream_decoder_set_metadata_respond (dec->stream_decoder,
       FLAC__METADATA_TYPE_VORBIS_COMMENT);
+  FLAC__stream_decoder_set_metadata_respond (dec->stream_decoder,
+      FLAC__METADATA_TYPE_PICTURE);
 #endif
 }
 
@@ -349,10 +432,13 @@ gst_flac_dec_update_metadata (GstFlacDec * flacdec,
   GstTagList *list;
   guint num, i;
 
-  list = gst_tag_list_new ();
+  if (flacdec->tags)
+    list = flacdec->tags;
+  else
+    flacdec->tags = list = gst_tag_list_new ();
 
   num = metadata->data.vorbis_comment.num_comments;
-  GST_DEBUG ("%u tag(s) found", num);
+  GST_DEBUG_OBJECT (flacdec, "%u tag(s) found", num);
 
   for (i = 0; i < num; ++i) {
     gchar *vc, *name, *value;
@@ -361,7 +447,7 @@ gst_flac_dec_update_metadata (GstFlacDec * flacdec,
         metadata->data.vorbis_comment.comments[i].length);
 
     if (gst_tag_parse_extended_comment (vc, &name, NULL, &value, TRUE)) {
-      GST_DEBUG ("%s : %s", name, value);
+      GST_DEBUG_OBJECT (flacdec, "%s : %s", name, value);
       gst_vorbis_tag_add (list, name, value);
       g_free (name);
       g_free (value);
@@ -370,11 +456,6 @@ gst_flac_dec_update_metadata (GstFlacDec * flacdec,
     g_free (vc);
   }
 
-  gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
-      GST_TAG_AUDIO_CODEC, "FLAC", NULL);
-
-  gst_element_found_tags_for_pad (GST_ELEMENT (flacdec), flacdec->srcpad, list);
-
   return TRUE;
 }
 
@@ -450,7 +531,8 @@ gst_flac_dec_scan_got_frame (GstFlacDec * flacdec, guint8 * data, guint size,
   ss = (data[3] & 0x0F) >> 1;   /* sample size marker */
   pb = (data[3] & 0x01);        /* padding bit        */
 
-  GST_LOG ("got sync, bs=%x,sr=%x,ca=%x,ss=%x,pb=%x", bs, sr, ca, ss, pb);
+  GST_LOG_OBJECT (flacdec,
+      "got sync, bs=%x,sr=%x,ca=%x,ss=%x,pb=%x", bs, sr, ca, ss, pb);
 
   if (sr == 0x0F || sr == 0x01 || sr == 0x02 || sr == 0x03 ||
       ca >= 0x0B || ss == 0x03 || ss == 0x07) {
@@ -503,6 +585,7 @@ static void
 gst_flac_dec_scan_for_last_block (GstFlacDec * flacdec, gint64 * samples)
 {
   GstFormat format = GST_FORMAT_BYTES;
+
   gint64 file_size, offset;
 
   GST_INFO_OBJECT (flacdec, "total number of samples unknown, scanning file");
@@ -550,6 +633,38 @@ gst_flac_dec_scan_for_last_block (GstFlacDec * flacdec, gint64 * samples)
   }
 }
 
+/* FIXME: remove ifndef once we depend on flac >= 1.2.x */
+#ifndef LEGACY_FLAC
+static void
+gst_flac_extract_picture_buffer (GstFlacDec * dec,
+    const FLAC__StreamMetadata * metadata)
+{
+  FLAC__StreamMetadata_Picture picture;
+  GstTagList *tags;
+
+  g_return_if_fail (metadata->type == FLAC__METADATA_TYPE_PICTURE);
+
+  GST_LOG_OBJECT (dec, "Got PICTURE block");
+  picture = metadata->data.picture;
+
+  GST_DEBUG_OBJECT (dec, "declared MIME type is: '%s'",
+      GST_STR_NULL (picture.mime_type));
+  GST_DEBUG_OBJECT (dec, "image data is %u bytes", picture.data_length);
+
+  tags = gst_tag_list_new ();
+
+  gst_tag_list_add_id3_image (tags, (guint8 *) picture.data,
+      picture.data_length, picture.type);
+
+  if (!gst_tag_list_is_empty (tags)) {
+    gst_element_found_tags_for_pad (GST_ELEMENT (dec), dec->srcpad, tags);
+  } else {
+    GST_DEBUG_OBJECT (dec, "problem parsing PICTURE block, skipping");
+    gst_tag_list_free (tags);
+  }
+}
+#endif /* LEGACY_FLAC */
+
 static void
 gst_flac_dec_metadata_callback (GstFlacDec * flacdec,
     const FLAC__StreamMetadata * metadata)
@@ -563,6 +678,7 @@ gst_flac_dec_metadata_callback (GstFlacDec * flacdec,
       flacdec->min_blocksize = metadata->data.stream_info.min_blocksize;
       flacdec->max_blocksize = metadata->data.stream_info.max_blocksize;
       flacdec->sample_rate = metadata->data.stream_info.sample_rate;
+      flacdec->depth = metadata->data.stream_info.bits_per_sample;
 
       GST_DEBUG_OBJECT (flacdec, "blocksize: min=%u, max=%u",
           flacdec->min_blocksize, flacdec->max_blocksize);
@@ -575,13 +691,34 @@ gst_flac_dec_metadata_callback (GstFlacDec * flacdec,
       GST_DEBUG_OBJECT (flacdec, "total samples = %" G_GINT64_FORMAT, samples);
 
       if (samples > 0) {
+        gint64 duration;
+
         gst_segment_set_duration (&flacdec->segment, GST_FORMAT_DEFAULT,
             samples);
-        if (flacdec->segment.stop == -1)
-          flacdec->segment.stop = samples;
+
+        /* convert duration to time */
+        duration = gst_util_uint64_scale_int (samples, GST_SECOND,
+            flacdec->sample_rate);
+
+        /* fixme, at this time we could seek to the queued seek event if we have
+         * any */
+        if (flacdec->start_segment)
+          gst_event_unref (flacdec->start_segment);
+        flacdec->start_segment =
+            gst_event_new_new_segment_full (FALSE,
+            flacdec->segment.rate, flacdec->segment.applied_rate,
+            GST_FORMAT_TIME, 0, duration, 0);
       }
       break;
     }
+
+/* FIXME: remove ifndef once we depend on flac >= 1.2.x */
+#ifndef LEGACY_FLAC
+    case FLAC__METADATA_TYPE_PICTURE:{
+      gst_flac_extract_picture_buffer (flacdec, metadata);
+      break;
+    }
+#endif
     case FLAC__METADATA_TYPE_VORBIS_COMMENT:
       gst_flac_dec_update_metadata (flacdec, metadata);
       break;
@@ -665,7 +802,7 @@ gst_flac_dec_seek (const FLAC__StreamDecoder * decoder,
 
   flacdec = GST_FLAC_DEC (client_data);
 
-  GST_DEBUG ("seek %" G_GINT64_FORMAT, position);
+  GST_DEBUG_OBJECT (flacdec, "seek %" G_GINT64_FORMAT, position);
   flacdec->offset = position;
 
 #ifdef LEGACY_FLAC
@@ -691,7 +828,7 @@ gst_flac_dec_tell (const FLAC__StreamDecoder * decoder,
 
   *position = flacdec->offset;
 
-  GST_DEBUG ("tell %" G_GINT64_FORMAT, *position);
+  GST_DEBUG_OBJECT (flacdec, "tell %" G_GINT64_FORMAT, *position);
 
 #ifdef LEGACY_FLAC
   return FLAC__SEEKABLE_STREAM_DECODER_TELL_STATUS_OK;
@@ -734,7 +871,7 @@ gst_flac_dec_length (const FLAC__StreamDecoder * decoder,
 
   *length = len;
 
-  GST_DEBUG ("length %" G_GINT64_FORMAT, *length);
+  GST_DEBUG_OBJECT (flacdec, "length %" G_GINT64_FORMAT, *length);
 
 #ifdef LEGACY_FLAC
   return FLAC__SEEKABLE_STREAM_DECODER_LENGTH_STATUS_OK;
@@ -768,7 +905,8 @@ gst_flac_dec_eof (const FLAC__StreamDecoder * decoder, void *client_data)
   fmt = GST_FORMAT_BYTES;
   if (gst_pad_query_duration (peer, &fmt, &len) && fmt == GST_FORMAT_BYTES &&
       len != -1 && flacdec->offset >= len) {
-    GST_DEBUG ("offset=%" G_GINT64_FORMAT ", len=%" G_GINT64_FORMAT
+    GST_DEBUG_OBJECT (flacdec,
+        "offset=%" G_GINT64_FORMAT ", len=%" G_GINT64_FORMAT
         ", returning EOF", flacdec->offset, len);
     ret = TRUE;
   }
@@ -789,6 +927,7 @@ gst_flac_dec_read_seekable (const FLAC__StreamDecoder * decoder,
 #endif
 {
   GstFlacDec *flacdec;
+
   GstBuffer *buf;
 
   flacdec = GST_FLAC_DEC (client_data);
@@ -801,7 +940,7 @@ gst_flac_dec_read_seekable (const FLAC__StreamDecoder * decoder,
     return FLAC__STREAM_DECODER_READ_STATUS_ABORT;
 #endif
 
-  GST_DEBUG ("Read %d bytes at %" G_GUINT64_FORMAT,
+  GST_DEBUG_OBJECT (flacdec, "Read %d bytes at %" G_GUINT64_FORMAT,
       GST_BUFFER_SIZE (buf), flacdec->offset);
   memcpy (buffer, GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
   *bytes = GST_BUFFER_SIZE (buf);
@@ -836,7 +975,7 @@ gst_flac_dec_read_stream (const FLAC__StreamDecoder * decoder,
   }
 
   GST_LOG_OBJECT (dec, "feeding %u bytes to decoder (available=%u, bytes=%u)",
-      len, gst_adapter_available (dec->adapter), *bytes);
+      len, gst_adapter_available (dec->adapter), (guint) * bytes);
   memcpy (buffer, gst_adapter_peek (dec->adapter, len), len);
   *bytes = len;
 
@@ -856,6 +995,7 @@ gst_flac_dec_write (GstFlacDec * flacdec, const FLAC__Frame * frame,
   guint channels = frame->header.channels;
   guint samples = frame->header.blocksize;
   guint j, i;
+  GstClockTime next;
 
   switch (depth) {
     case 8:
@@ -869,15 +1009,34 @@ gst_flac_dec_write (GstFlacDec * flacdec, const FLAC__Frame * frame,
     case 24:
     case 32:
       width = 32;
+      break;
+    case 0:
+      if (flacdec->depth < 4 || flacdec->depth > 32) {
+        GST_ERROR_OBJECT (flacdec, "unsupported depth %d from STREAMINFO",
+            flacdec->depth);
+        ret = GST_FLOW_ERROR;
+        goto done;
+      }
+
+      depth = flacdec->depth;
+      if (depth < 9)
+        width = 8;
+      else if (depth < 17)
+        width = 16;
+      else
+        width = 32;
+
       break;
     default:
-      g_assert_not_reached ();
+      GST_ERROR_OBJECT (flacdec, "unsupported depth %d", depth);
+      ret = GST_FLOW_ERROR;
+      goto done;
   }
 
   if (!GST_PAD_CAPS (flacdec->srcpad)) {
     GstCaps *caps;
 
-    GST_DEBUG ("Negotiating %d Hz @ %d channels",
+    GST_DEBUG_OBJECT (flacdec, "Negotiating %d Hz @ %d channels",
         frame->header.sample_rate, channels);
 
     caps = gst_caps_new_simple ("audio/x-raw-int",
@@ -888,25 +1047,34 @@ gst_flac_dec_write (GstFlacDec * flacdec, const FLAC__Frame * frame,
         "rate", G_TYPE_INT, frame->header.sample_rate,
         "channels", G_TYPE_INT, channels, NULL);
 
-    if (!gst_pad_set_caps (flacdec->srcpad, caps)) {
-      GST_ELEMENT_ERROR (flacdec, CORE, NEGOTIATION, (NULL),
-          ("Failed to negotiate caps %" GST_PTR_FORMAT, caps));
-      flacdec->last_flow = GST_FLOW_ERROR;
-      gst_caps_unref (caps);
-      return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
-    }
+    if (channels > 2) {
+      GstStructure *s = gst_caps_get_structure (caps, 0);
 
-    gst_caps_unref (caps);
+      gst_audio_set_channel_positions (s, channel_positions[channels - 1]);
+    }
 
     flacdec->depth = depth;
     flacdec->width = width;
     flacdec->channels = channels;
     flacdec->sample_rate = frame->header.sample_rate;
+
+    gst_pad_set_caps (flacdec->srcpad, caps);
+    gst_caps_unref (caps);
   }
 
-  if (flacdec->need_newsegment) {
-    gst_flac_dec_send_newsegment (flacdec, FALSE);
-    flacdec->need_newsegment = FALSE;
+  if (flacdec->close_segment) {
+    gst_pad_push_event (flacdec->srcpad, flacdec->close_segment);
+    flacdec->close_segment = NULL;
+  }
+  if (flacdec->start_segment) {
+    gst_pad_push_event (flacdec->srcpad, flacdec->start_segment);
+    flacdec->start_segment = NULL;
+  }
+
+  if (flacdec->tags) {
+    gst_element_found_tags_for_pad (GST_ELEMENT (flacdec), flacdec->srcpad,
+        flacdec->tags);
+    flacdec->tags = NULL;
   }
 
   ret = gst_pad_alloc_buffer_and_set_caps (flacdec->srcpad,
@@ -914,7 +1082,8 @@ gst_flac_dec_write (GstFlacDec * flacdec, const FLAC__Frame * frame,
       GST_PAD_CAPS (flacdec->srcpad), &outbuf);
 
   if (ret != GST_FLOW_OK) {
-    GST_DEBUG ("gst_pad_alloc_buffer() returned %s", gst_flow_get_name (ret));
+    GST_DEBUG_OBJECT (flacdec, "gst_pad_alloc_buffer() returned %s",
+        gst_flow_get_name (ret));
     goto done;
   }
 
@@ -929,11 +1098,13 @@ gst_flac_dec_write (GstFlacDec * flacdec, const FLAC__Frame * frame,
       gst_util_uint64_scale_int (flacdec->segment.last_stop, GST_SECOND,
       frame->header.sample_rate);
 
-  GST_BUFFER_DURATION (outbuf) =
-      gst_util_uint64_scale_int (samples, GST_SECOND,
-      frame->header.sample_rate);
+  /* get next timestamp to calculate the duration */
+  next = gst_util_uint64_scale_int (flacdec->segment.last_stop + samples,
+      GST_SECOND, frame->header.sample_rate);
 
-  if (depth == 8) {
+  GST_BUFFER_DURATION (outbuf) = next - GST_BUFFER_TIMESTAMP (outbuf);
+
+  if (width == 8) {
     gint8 *outbuffer = (gint8 *) GST_BUFFER_DATA (outbuf);
 
     for (i = 0; i < samples; i++) {
@@ -941,7 +1112,7 @@ gst_flac_dec_write (GstFlacDec * flacdec, const FLAC__Frame * frame,
         *outbuffer++ = (gint8) buffer[j][i];
       }
     }
-  } else if (depth == 12 || depth == 16) {
+  } else if (width == 16) {
     gint16 *outbuffer = (gint16 *) GST_BUFFER_DATA (outbuf);
 
     for (i = 0; i < samples; i++) {
@@ -949,7 +1120,7 @@ gst_flac_dec_write (GstFlacDec * flacdec, const FLAC__Frame * frame,
         *outbuffer++ = (gint16) buffer[j][i];
       }
     }
-  } else if (depth == 20 || depth == 24 || depth == 32) {
+  } else if (width == 32) {
     gint32 *outbuffer = (gint32 *) GST_BUFFER_DATA (outbuf);
 
     for (i = 0; i < samples; i++) {
@@ -962,22 +1133,29 @@ gst_flac_dec_write (GstFlacDec * flacdec, const FLAC__Frame * frame,
   }
 
   if (!flacdec->seeking) {
-    GST_DEBUG ("pushing %d samples at offset %" G_GINT64_FORMAT
+    GST_DEBUG_OBJECT (flacdec, "pushing %d samples at offset %" G_GINT64_FORMAT
         " (%" GST_TIME_FORMAT " + %" GST_TIME_FORMAT ")",
         samples, GST_BUFFER_OFFSET (outbuf),
         GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)),
         GST_TIME_ARGS (GST_BUFFER_DURATION (outbuf)));
 
+    if (flacdec->discont) {
+      outbuf = gst_buffer_make_metadata_writable (outbuf);
+      GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
+      flacdec->discont = FALSE;
+    }
     ret = gst_pad_push (flacdec->srcpad, outbuf);
   } else {
-    GST_DEBUG ("not pushing %d samples at offset %" G_GINT64_FORMAT
+    GST_DEBUG_OBJECT (flacdec,
+        "not pushing %d samples at offset %" G_GINT64_FORMAT
         " (in seek)", samples, GST_BUFFER_OFFSET (outbuf));
     gst_buffer_unref (outbuf);
     ret = GST_FLOW_OK;
   }
 
   if (ret != GST_FLOW_OK) {
-    GST_DEBUG ("gst_pad_push() returned %s", gst_flow_get_name (ret));
+    GST_DEBUG_OBJECT (flacdec, "gst_pad_push() returned %s",
+        gst_flow_get_name (ret));
   }
 
 done:
@@ -1015,7 +1193,6 @@ static void
 gst_flac_dec_loop (GstPad * sinkpad)
 {
   GstFlacDec *flacdec;
-
 #ifdef LEGACY_FLAC
   FLAC__SeekableStreamDecoderState s;
 #else
@@ -1028,12 +1205,13 @@ gst_flac_dec_loop (GstPad * sinkpad)
   GST_LOG_OBJECT (flacdec, "entering loop");
 
   if (flacdec->init) {
-    GST_DEBUG_OBJECT (flacdec, "initializing decoder");
 #ifdef LEGACY_FLAC
+    GST_DEBUG_OBJECT (flacdec, "initializing legacy decoder");
     s = FLAC__seekable_stream_decoder_init (flacdec->seekable_decoder);
     if (s != FLAC__SEEKABLE_STREAM_DECODER_OK)
       goto analyze_state;
 #else
+    GST_DEBUG_OBJECT (flacdec, "initializing new decoder");
     is = FLAC__stream_decoder_init_stream (flacdec->seekable_decoder,
         gst_flac_dec_read_seekable, gst_flac_dec_seek, gst_flac_dec_tell,
         gst_flac_dec_length, gst_flac_dec_eof, gst_flac_dec_write_stream,
@@ -1177,14 +1355,13 @@ segment_done_and_pause:
 
     goto pause;
   }
-
 eos_and_pause:
   {
     GST_DEBUG_OBJECT (flacdec, "sending EOS event");
+    flacdec->running = FALSE;
     gst_pad_push_event (flacdec->srcpad, gst_event_new_eos ());
     /* fall through to pause */
   }
-
 pause:
   {
     GST_DEBUG_OBJECT (flacdec, "pausing");
@@ -1212,8 +1389,11 @@ gst_flac_dec_sink_event (GstPad * pad, GstEvent * event)
     }
     case GST_EVENT_NEWSEGMENT:{
       GstFormat fmt;
+
       gboolean update;
+
       gdouble rate, applied_rate;
+
       gint64 cur, stop, time;
 
       gst_event_parse_new_segment_full (event, &update, &rate, &applied_rate,
@@ -1225,7 +1405,6 @@ gst_flac_dec_sink_event (GstPad * pad, GstEvent * event)
         GST_DEBUG_OBJECT (dec, "newsegment event in TIME format => framed");
         dec->framed = TRUE;
         res = gst_pad_push_event (dec->srcpad, event);
-        dec->need_newsegment = FALSE;
 
         /* this won't work for the first newsegment event though ... */
         if (gst_flac_dec_convert_src (dec->srcpad, GST_FORMAT_TIME, cur,
@@ -1242,7 +1421,6 @@ gst_flac_dec_sink_event (GstPad * pad, GstEvent * event)
         GST_DEBUG_OBJECT (dec, "newsegment event in %s format => not framed",
             gst_format_get_name (fmt));
         dec->framed = FALSE;
-        dec->need_newsegment = TRUE;
         gst_event_unref (event);
         res = TRUE;
       }
@@ -1352,6 +1530,13 @@ gst_flac_dec_chain (GstPad * pad, GstBuffer * buf)
         GST_DEBUG_OBJECT (dec, "process_single failed");
         break;
       }
+
+      if (FLAC__stream_decoder_get_state (dec->stream_decoder) ==
+          FLAC__STREAM_DECODER_ABORTED) {
+        GST_WARNING_OBJECT (dec, "Read callback caused internal abort");
+        dec->last_flow = GST_FLOW_ERROR;
+        break;
+      }
     }
   } else if (dec->framed && got_audio_frame) {
     /* framed - there should always be enough data to decode something */
@@ -1444,6 +1629,7 @@ gst_flac_dec_sink_query (GstPad * pad, GstQuery * query)
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_CONVERT:{
       GstFormat src_fmt, dest_fmt;
+
       gint64 src_val, dest_val;
 
       gst_query_parse_convert (query, &src_fmt, &src_val, &dest_fmt, NULL);
@@ -1563,6 +1749,7 @@ gst_flac_dec_src_query (GstPad * pad, GstQuery * query)
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_POSITION:{
       GstFormat fmt;
+
       gint64 pos;
 
       gst_query_parse_position (query, &fmt, NULL);
@@ -1594,6 +1781,7 @@ gst_flac_dec_src_query (GstPad * pad, GstQuery * query)
 
     case GST_QUERY_DURATION:{
       GstFormat fmt;
+
       gint64 len;
 
       gst_query_parse_duration (query, &fmt, NULL);
@@ -1637,6 +1825,7 @@ gst_flac_dec_src_query (GstPad * pad, GstQuery * query)
 
     case GST_QUERY_CONVERT:{
       GstFormat src_fmt, dest_fmt;
+
       gint64 src_val, dest_val;
 
       gst_query_parse_convert (query, &src_fmt, &src_val, &dest_fmt, NULL);
@@ -1667,37 +1856,6 @@ done:
   return res;
 }
 
-static void
-gst_flac_dec_send_newsegment (GstFlacDec * flacdec, gboolean update)
-{
-  GstSegment *s = &flacdec->segment;
-  GstFormat target_format = GST_FORMAT_TIME;
-  gint64 stop_time = -1;
-  gint64 start_time = 0;
-
-  /* segment is in DEFAULT format, but we want to send a TIME newsegment */
-  if (!gst_flac_dec_convert_src (flacdec->srcpad, GST_FORMAT_DEFAULT,
-          s->start, &target_format, &start_time)) {
-    GST_WARNING_OBJECT (flacdec, "failed to convert segment start %lld to TIME",
-        s->start);
-    return;
-  }
-
-  if (s->stop != -1 && !gst_flac_dec_convert_src (flacdec->srcpad,
-          GST_FORMAT_DEFAULT, s->stop, &target_format, &stop_time)) {
-    GST_WARNING_OBJECT (flacdec, "failed to convert segment stop to TIME");
-    return;
-  }
-
-  GST_DEBUG_OBJECT (flacdec, "sending newsegment from %" GST_TIME_FORMAT
-      " to %" GST_TIME_FORMAT, GST_TIME_ARGS (start_time),
-      GST_TIME_ARGS (stop_time));
-
-  gst_pad_push_event (flacdec->srcpad,
-      gst_event_new_new_segment (update, s->rate, GST_FORMAT_TIME,
-          start_time, stop_time, start_time));
-}
-
 static gboolean
 gst_flac_dec_handle_seek_event (GstFlacDec * flacdec, GstEvent * event)
 {
@@ -1710,7 +1868,7 @@ gst_flac_dec_handle_seek_event (GstFlacDec * flacdec, GstEvent * event)
   gboolean only_update = FALSE;
   gboolean flush;
   gdouble rate;
-  gint64 start;
+  gint64 start, last_stop;
   gint64 stop;
 
   if (flacdec->seekable_decoder == NULL) {
@@ -1722,12 +1880,14 @@ gst_flac_dec_handle_seek_event (GstFlacDec * flacdec, GstEvent * event)
       &start, &stop_type, &stop);
 
   if (seek_format != GST_FORMAT_DEFAULT && seek_format != GST_FORMAT_TIME) {
-    GST_DEBUG ("seeking is only supported in TIME or DEFAULT format");
+    GST_DEBUG_OBJECT (flacdec,
+        "seeking is only supported in TIME or DEFAULT format");
     return FALSE;
   }
 
   if (rate < 0.0) {
-    GST_DEBUG ("only forward playback supported, rate %f not allowed", rate);
+    GST_DEBUG_OBJECT (flacdec,
+        "only forward playback supported, rate %f not allowed", rate);
     return FALSE;
   }
 
@@ -1737,14 +1897,14 @@ gst_flac_dec_handle_seek_event (GstFlacDec * flacdec, GstEvent * event)
     if (start_type != GST_SEEK_TYPE_NONE &&
         !gst_flac_dec_convert_src (flacdec->srcpad, seek_format, start,
             &target_format, &start)) {
-      GST_DEBUG ("failed to convert start to DEFAULT format");
+      GST_DEBUG_OBJECT (flacdec, "failed to convert start to DEFAULT format");
       return FALSE;
     }
 
     if (stop_type != GST_SEEK_TYPE_NONE &&
         !gst_flac_dec_convert_src (flacdec->srcpad, seek_format, stop,
             &target_format, &stop)) {
-      GST_DEBUG ("failed to convert stop to DEFAULT format");
+      GST_DEBUG_OBJECT (flacdec, "failed to convert stop to DEFAULT format");
       return FALSE;
     }
   }
@@ -1752,87 +1912,154 @@ gst_flac_dec_handle_seek_event (GstFlacDec * flacdec, GstEvent * event)
   flush = ((seek_flags & GST_SEEK_FLAG_FLUSH) == GST_SEEK_FLAG_FLUSH);
 
   if (flush) {
+    /* flushing seek, clear the pipeline of stuff, we need a newsegment after
+     * this. */
+    GST_DEBUG_OBJECT (flacdec, "flushing");
     gst_pad_push_event (flacdec->srcpad, gst_event_new_flush_start ());
   } else {
+    /* non flushing seek, pause the task */
+    GST_DEBUG_OBJECT (flacdec, "stopping task");
     gst_pad_stop_task (flacdec->sinkpad);
   }
 
+  /* acquire the stream lock, this either happens when the streaming thread
+   * stopped because of the flush or when the task is paused after the loop
+   * function finished an iteration, which can never happen when it's blocked
+   * downstream in PAUSED, for example */
   GST_PAD_STREAM_LOCK (flacdec->sinkpad);
 
-  /* operate on segment copy until we know the seek worked */
+  /* operate on segment copy until we know the seek worked. The idea is that
+   * when the seek fails, we want to continue with what we were doing without
+   * having the segment being updated. */
   segment = flacdec->segment;
 
-  if (segment.duration > 0) {
-    if (start != -1)
-      start = CLAMP (start, 0, segment.duration - 1);
-    if (stop != -1)
-      stop = CLAMP (stop, 0, segment.duration - 1);
-  }
-
+  /* update the segment with the seek values, last_stop will contain the new
+   * position we should seek to */
   gst_segment_set_seek (&segment, rate, GST_FORMAT_DEFAULT,
       seek_flags, start_type, start, stop_type, stop, &only_update);
 
-  GST_DEBUG ("configured segment: [%" G_GINT64_FORMAT "-%" G_GINT64_FORMAT
+  GST_DEBUG_OBJECT (flacdec,
+      "configured segment: [%" G_GINT64_FORMAT "-%" G_GINT64_FORMAT
       "] = [%" GST_TIME_FORMAT "-%" GST_TIME_FORMAT "]",
       segment.start, segment.stop,
       GST_TIME_ARGS (segment.start * GST_SECOND / flacdec->sample_rate),
       GST_TIME_ARGS (segment.stop * GST_SECOND / flacdec->sample_rate));
 
   GST_DEBUG_OBJECT (flacdec, "performing seek to sample %" G_GINT64_FORMAT,
-      segment.start);
+      segment.last_stop);
 
+  /* mark ourselves as seeking because the above lines will trigger some
+   * callbacks that need to behave differently when seeking */
   flacdec->seeking = TRUE;
 
   seek_ok =
 #ifdef  LEGACY_FLAC
       FLAC__seekable_stream_decoder_seek_absolute (flacdec->seekable_decoder,
-      segment.start);
+      segment.last_stop);
 #else
       FLAC__stream_decoder_seek_absolute (flacdec->seekable_decoder,
-      segment.start);
+      segment.last_stop);
 #endif
 
   flacdec->seeking = FALSE;
 
-  if (flush) {
-    gst_pad_push_event (flacdec->srcpad, gst_event_new_flush_stop ());
+  if (!seek_ok) {
+    /* seek failed, segment was not updated and we start streaming again with
+     * the previous segment values */
+    GST_WARNING_OBJECT (flacdec, "seek failed");
+  } else if (!flush && flacdec->running) {
+    /* we are running the current segment and doing a non-flushing seek, 
+     * close the segment first based on the last_stop. */
+    GST_DEBUG_OBJECT (flacdec, "closing running segment %" G_GINT64_FORMAT
+        " to %" G_GINT64_FORMAT, segment.start, segment.last_stop);
+
+    /* convert the old segment values to time to close the old segment */
+    start = gst_util_uint64_scale_int (flacdec->segment.start, GST_SECOND,
+        flacdec->sample_rate);
+    last_stop =
+        gst_util_uint64_scale_int (flacdec->segment.last_stop, GST_SECOND,
+        flacdec->sample_rate);
+
+    /* queue the segment for sending in the stream thread, start and time are
+     * always the same. */
+    if (flacdec->close_segment)
+      gst_event_unref (flacdec->close_segment);
+    flacdec->close_segment =
+        gst_event_new_new_segment_full (TRUE,
+        flacdec->segment.rate, flacdec->segment.applied_rate, GST_FORMAT_TIME,
+        start, last_stop, start);
   }
 
   if (seek_ok) {
+    /* seek succeeded, copy new segment values */
     flacdec->segment = segment;
-    gst_flac_dec_send_newsegment (flacdec, FALSE);
-    flacdec->segment.last_stop = segment.start;
-
     GST_DEBUG_OBJECT (flacdec, "seek successful");
-
-    /* notify start of new segment */
-    if ((flacdec->segment.flags & GST_SEEK_FLAG_SEGMENT) != 0) {
-      GstMessage *msg;
-      gint64 start_time = 0;
-
-      start_time = gst_util_uint64_scale_int (segment.start, GST_SECOND,
-          flacdec->sample_rate);
-
-      msg = gst_message_new_segment_start (GST_OBJECT (flacdec),
-          GST_FORMAT_TIME, start_time);
-      gst_element_post_message (GST_ELEMENT (flacdec), msg);
-    }
-  } else {
-    GST_WARNING_OBJECT (flacdec, "seek failed");
   }
 
+  /* convert the (new) segment values to time, we will need them to generate the
+   * new segment events. */
+  start = gst_util_uint64_scale_int (flacdec->segment.start, GST_SECOND,
+      flacdec->sample_rate);
+  last_stop = gst_util_uint64_scale_int (flacdec->segment.last_stop, GST_SECOND,
+      flacdec->sample_rate);
+
+  /* for deriving a stop position for the playback segment from the seek
+   * segment, we must take the duration when the stop is not set */
+  if (flacdec->segment.stop != -1)
+    stop = gst_util_uint64_scale_int (flacdec->segment.stop, GST_SECOND,
+        flacdec->sample_rate);
+  else
+    stop = gst_util_uint64_scale_int (flacdec->segment.duration, GST_SECOND,
+        flacdec->sample_rate);
+
+  /* notify start of new segment when we were asked to do so. */
+  if (segment.flags & GST_SEEK_FLAG_SEGMENT) {
+    /* last_stop contains the position we start from */
+    gst_element_post_message (GST_ELEMENT (flacdec),
+        gst_message_new_segment_start (GST_OBJECT (flacdec),
+            GST_FORMAT_TIME, last_stop));
+  }
+
+  /* if the seek was ok or (when it failed) we are flushing, we need to send out
+   * a new segment. If we did not flush and the seek failed, we simply do
+   * nothing here and continue where we were. */
+  if (seek_ok || flush) {
+    GST_DEBUG_OBJECT (flacdec, "Creating newsegment from %" G_GINT64_FORMAT
+        " to %" G_GINT64_FORMAT, segment.last_stop, stop);
+    /* now replace the old segment so that we send it in the stream thread the
+     * next time it is scheduled. */
+    if (flacdec->start_segment)
+      gst_event_unref (flacdec->start_segment);
+    flacdec->start_segment =
+        gst_event_new_new_segment_full (FALSE,
+        segment.rate, segment.applied_rate, GST_FORMAT_TIME,
+        last_stop, stop, last_stop);
+  }
+
+  /* prepare for streaming again, start with a flush stop when we issued a flush
+   * start before. */
+  if (flush) {
+    GST_DEBUG_OBJECT (flacdec, "flushing stop");
+    gst_pad_push_event (flacdec->srcpad, gst_event_new_flush_stop ());
+  }
+
+  /* we'll generate a discont on the next buffer */
+  flacdec->discont = TRUE;
+  /* the task is running again now */
+  flacdec->running = TRUE;
   gst_pad_start_task (flacdec->sinkpad,
       (GstTaskFunction) gst_flac_dec_loop, flacdec->sinkpad);
 
   GST_PAD_STREAM_UNLOCK (flacdec->sinkpad);
 
-  return TRUE;
+  return seek_ok;
 }
 
 static gboolean
 gst_flac_dec_src_event (GstPad * pad, GstEvent * event)
 {
   gboolean res = TRUE;
+
   GstFlacDec *flacdec = GST_FLAC_DEC (gst_pad_get_parent (pad));
 
   switch (GST_EVENT_TYPE (event)) {
@@ -1872,13 +2099,16 @@ gst_flac_dec_sink_activate_push (GstPad * sinkpad, gboolean active)
 {
   GstFlacDec *dec = GST_FLAC_DEC (GST_OBJECT_PARENT (sinkpad));
 
-  gst_flac_dec_setup_stream_decoder (dec);
+  if (active)
+    gst_flac_dec_setup_stream_decoder (dec);
   return TRUE;
 }
 
 static gboolean
 gst_flac_dec_sink_activate_pull (GstPad * sinkpad, gboolean active)
 {
+  gboolean res;
+
   if (active) {
     GstFlacDec *flacdec;
 
@@ -1886,23 +2116,25 @@ gst_flac_dec_sink_activate_pull (GstPad * sinkpad, gboolean active)
 
     flacdec->offset = 0;
     gst_flac_dec_setup_seekable_decoder (flacdec);
-    return gst_pad_start_task (sinkpad, (GstTaskFunction) gst_flac_dec_loop,
+    flacdec->running = TRUE;
+
+    res = gst_pad_start_task (sinkpad, (GstTaskFunction) gst_flac_dec_loop,
         sinkpad);
   } else {
-    return gst_pad_stop_task (sinkpad);
+    res = gst_pad_stop_task (sinkpad);
   }
+  return res;
 }
 
 static GstStateChangeReturn
 gst_flac_dec_change_state (GstElement * element, GstStateChange transition)
 {
   GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
+
   GstFlacDec *flacdec = GST_FLAC_DEC (element);
 
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      flacdec->segment.last_stop = 0;
-      flacdec->need_newsegment = TRUE;
       flacdec->seeking = FALSE;
       flacdec->channels = 0;
       flacdec->depth = 0;
