@@ -28,6 +28,8 @@
 #include <pulse/pulseaudio.h>
 #include <pulse/thread-mainloop.h>
 
+#include "pulseprobe.h"
+
 G_BEGIN_DECLS
 
 #define GST_TYPE_PULSESINK \
@@ -57,7 +59,16 @@ struct _GstPulseSink
 
   pa_sample_spec sample_spec;
 
-  int operation_success;
+  GstPulseProbe *probe;
+
+  gdouble volume;
+  gboolean volume_set;
+
+  gchar *device_description;
+
+  gboolean operation_success;
+  gboolean did_reset, in_write;
+  gint notify;
 };
 
 struct _GstPulseSinkClass
