@@ -20,33 +20,26 @@
 
 /**
  * SECTION:element-apedemux
- * @short_description: reads tag information from APE tag data blocks and
- * outputs them as GStreamer tag messages and events.
  *
- * <refsect2>
- * <para>
  * apedemux accepts data streams with APE tags at the start or at the end
  * (or both). The mime type of the data between the tag blocks is detected
  * using typefind functions, and the appropriate output mime type set on
- * outgoing buffers. 
- * </para>
- * <para>
+ * outgoing buffers.
+ *
  * The element is only able to read APE tags at the end of a stream from
  * a seekable stream, ie. when get_range mode is supported by the upstream
  * elements. If get_range operation is available, apedemux makes it available
  * downstream. This means that elements which require get_range mode, such as
  * wavparse or musepackdec, can operate on files containing APE tag
  * information.
- * </para>
+ *
+ * <refsect2>
  * <title>Example launch line</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch -t filesrc location=file.mpc ! apedemux ! fakesink
- * </programlisting>
- * This pipeline should read any available APE tag information and output it.
+ * ]| This pipeline should read any available APE tag information and output it.
  * The contents of the file inside the APE tag regions should be detected, and
  * the appropriate mime type set on buffers produced from apedemux.
- * </para>
  * </refsect2>
  */
 #ifdef HAVE_CONFIG_H
@@ -109,17 +102,19 @@ gst_ape_demux_class_init (GstApeDemuxClass * klass)
 
   tagdemux_class->identify_tag = GST_DEBUG_FUNCPTR (gst_ape_demux_identify_tag);
   tagdemux_class->parse_tag = GST_DEBUG_FUNCPTR (gst_ape_demux_parse_tag);
+
+  /* no need for a merge function, the default behaviour to prefer start
+   * tags (APEv2) over end tags (usually APEv1, but could theoretically also
+   * be APEv2) is fine */
+
+  tagdemux_class->min_start_size = 32;
+  tagdemux_class->min_end_size = 32;
 }
 
 static void
 gst_ape_demux_init (GstApeDemux * apedemux, GstApeDemuxClass * gclass)
 {
-  GstTagDemux *tagdemux = GST_TAG_DEMUX (apedemux);
-
-  tagdemux->min_start_size = 32;
-  tagdemux->min_end_size = 32;
-
-  tagdemux->prefer_start_tag = TRUE;
+  /* nothing to do here */
 }
 
 static const struct _GstApeDemuxTagTableEntry
@@ -135,6 +130,7 @@ static const struct _GstApeDemuxTagTableEntry
   "title", GST_TAG_TITLE}, {
   "artist", GST_TAG_ARTIST}, {
   "album", GST_TAG_ALBUM}, {
+  "composer", GST_TAG_COMPOSER}, {
   "comment", GST_TAG_COMMENT}, {
   "comments", GST_TAG_COMMENT}, {
   "copyright", GST_TAG_COPYRIGHT}, {
