@@ -228,7 +228,7 @@ gst_shagadelictv_transform (GstBaseTransform * trans, GstBuffer * in,
 
   filter = GST_SHAGADELICTV (trans);
 
-  gst_buffer_stamp (out, in);
+  gst_buffer_copy_metadata (out, in, GST_BUFFER_COPY_TIMESTAMPS);
 
   src = (guint32 *) GST_BUFFER_DATA (in);
   dest = (guint32 *) GST_BUFFER_DATA (out);
