@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: t; c-basic-offset: 2 -*- */
-/* Copyright 2006 Tim-Philipp Müller <tim centricular net>
+/* Copyright 2006-2008 Tim-Philipp Müller <tim centricular net>
  * Copyright 2005 Jan Schmidt <thaytan@mad.scientist.com>
  * Copyright 2002,2003 Scott Wheeler <wheeler@kde.org> (portions from taglib)
  *
@@ -38,12 +38,15 @@ GST_DEBUG_CATEGORY_EXTERN (id3demux_debug);
 #define GST_CAT_DEFAULT (id3demux_debug)
 
 static gboolean parse_comment_frame (ID3TagsWorking * work);
+static gchar *parse_url_link_frame (ID3TagsWorking * work,
+    const gchar ** tag_name);
 static GArray *parse_text_identification_frame (ID3TagsWorking * work);
 static gchar *parse_user_text_identification_frame (ID3TagsWorking * work,
     const gchar ** tag_name);
 static gchar *parse_unique_file_identifier (ID3TagsWorking * work,
     const gchar ** tag_name);
 static gboolean parse_relative_volume_adjustment_two (ID3TagsWorking * work);
+static void parse_obsolete_tdat_frame (ID3TagsWorking * work);
 static gboolean id3v2_tag_to_taglist (ID3TagsWorking * work,
     const gchar * tag_name, const gchar * tag_str);
 /* Parse a single string into an array of gchar* */
@@ -60,8 +63,6 @@ static gboolean parse_picture_frame (ID3TagsWorking * work);
 #define ID3V2_ENCODING_UTF16BE 0x02
 #define ID3V2_ENCODING_UTF8    0x03
 
-extern guint read_synch_uint (guint8 * data, guint size);
-
 gboolean
 id3demux_id3v2_parse_frame (ID3TagsWorking * work)
 {
@@ -100,6 +101,7 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
   if (tag_name == NULL &&
       strncmp (work->frame_id, "RVA2", 4) != 0 &&
       strncmp (work->frame_id, "TXXX", 4) != 0 &&
+      strncmp (work->frame_id, "TDAT", 4) != 0 &&
       strncmp (work->frame_id, "UFID", 4) != 0) {
     return FALSE;
   }
@@ -152,13 +154,19 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
   }
 
   if (work->frame_id[0] == 'T') {
-    if (strcmp (work->frame_id, "TXXX") != 0) {
-      /* Text identification frame */
-      tag_fields = parse_text_identification_frame (work);
-    } else {
+    if (strcmp (work->frame_id, "TDAT") == 0) {
+      parse_obsolete_tdat_frame (work);
+      result = TRUE;
+    } else if (strcmp (work->frame_id, "TXXX") == 0) {
       /* Handle user text frame */
       tag_str = parse_user_text_identification_frame (work, &tag_name);
+    } else {
+      /* Text identification frame */
+      tag_fields = parse_text_identification_frame (work);
     }
+  } else if (work->frame_id[0] == 'W' && strcmp (work->frame_id, "WXXX") != 0) {
+    /* URL link frame: ISO-8859-1 encoded, one frame per tag */
+    tag_str = parse_url_link_frame (work, &tag_name);
   } else if (!strcmp (work->frame_id, "COMM")) {
     /* Comment */
     result = parse_comment_frame (work);
@@ -217,10 +225,10 @@ parse_comment_frame (ID3TagsWorking * work)
     return FALSE;
 
   encoding = work->parse_data[0];
-  language[0] = work->parse_data[1];
-  language[1] = work->parse_data[2];
-  language[2] = work->parse_data[3];
-  language[3] = 0;
+  language[0] = g_ascii_tolower (work->parse_data[1]);
+  language[1] = g_ascii_tolower (work->parse_data[2]);
+  language[2] = g_ascii_tolower (work->parse_data[3]);
+  language[3] = '\0';
 
   parse_split_strings (encoding, (gchar *) work->parse_data + 4,
       work->parse_size - 4, &fields);
@@ -242,7 +250,11 @@ parse_comment_frame (ID3TagsWorking * work)
       sscanf (description, "c%u", &dummy) != 1) {
     gchar *s;
 
-    if (language[0] != '\0') {
+    /* must be either an ISO-639-1 or ISO-639-2 language code */
+    if (language[0] != '\0' &&
+        g_ascii_isalpha (language[0]) &&
+        g_ascii_isalpha (language[1]) &&
+        (g_ascii_isalpha (language[2]) || language[2] == '\0')) {
       s = g_strdup_printf ("%s[%s]=%s", description, language, text);
     } else {
       s = g_strdup_printf ("%s=%s", description, text);
@@ -294,6 +306,63 @@ parse_text_identification_frame (ID3TagsWorking * work)
   return fields;
 }
 
+static gboolean
+link_is_known_license (const gchar * url)
+{
+  return g_str_has_prefix (url, "http://creativecommons.org/licenses/");
+}
+
+static gchar *
+parse_url_link_frame (ID3TagsWorking * work, const gchar ** tag_name)
+{
+  gsize len;
+  gchar *nul, *data, *link;
+
+  *tag_name = NULL;
+
+  if (work->parse_size == 0)
+    return NULL;
+
+  data = (gchar *) work->parse_data;
+  /* if there's more data then the string is long, we only want to parse the
+   * data up to the terminating zero to g_convert and ignore the rest, as
+   * per spec */
+  nul = memchr (data, '\0', work->parse_size);
+  if (nul != NULL) {
+    len = (gsize) (nul - data);
+  } else {
+    len = work->parse_size;
+  }
+
+  link = g_convert (data, len, "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
+
+  if (link == NULL || !gst_uri_is_valid (link)) {
+    GST_DEBUG ("Invalid URI in %s frame: %s", work->frame_id,
+        GST_STR_NULL (link));
+    g_free (link);
+    return NULL;
+  }
+
+  /* we don't know if it's a link to a page that explains the copyright
+   * situation, or a link that points to/represents a license, the ID3 spec
+   * does not separate those two things; for now only put known license URIs
+   * into GST_TAG_LICENSE_URI and everything else into GST_TAG_COPYRIGHT_URI */
+  if (strcmp (work->frame_id, "WCOP") == 0) {
+    if (link_is_known_license (link))
+      *tag_name = GST_TAG_LICENSE_URI;
+    else
+      *tag_name = GST_TAG_COPYRIGHT_URI;
+  } else if (strcmp (work->frame_id, "WOAF") == 0) {
+    /* can't be bothered to create a CONTACT_URI tag for this, so let's just
+     * put into into GST_TAG_CONTACT, which is where it ends up when reading
+     * the info from vorbis comments as well */
+    *tag_name = GST_TAG_CONTACT;
+  }
+
+  return link;
+}
+
+
 static gchar *
 parse_user_text_identification_frame (ID3TagsWorking * work,
     const gchar ** tag_name)
@@ -424,14 +493,11 @@ scan_encoded_string (guint8 encoding, gchar * data, gint data_size)
 static gboolean
 parse_picture_frame (ID3TagsWorking * work)
 {
-  GstBuffer *image = NULL;
-  GstCaps *image_caps = NULL;
-  gboolean is_pic_uri = FALSE;
   guint8 txt_encoding, pic_type;
   gchar *mime_str = NULL;
   gint len, datalen;
 
-  GST_LOG ("APIC frame");
+  GST_LOG ("APIC frame (ID3v2.%u)", ID3V2_VER_MAJOR (work->hdr.version));
 
   if (work->parse_size < 1 + 1 + 1 + 1 + 1)
     goto not_enough_data;
@@ -440,26 +506,28 @@ parse_picture_frame (ID3TagsWorking * work)
   ++work->parse_data;
   --work->parse_size;
 
-  if (!parse_id_string (work, &mime_str, &len, &datalen)) {
-    return FALSE;
+  /* Read image format; in early ID3v2 versions this is a fixed-length
+   * 3-character string without terminator; in later versions (>= 2.3.0)
+   * this is a NUL-terminated string of variable length */
+  if (ID3V2_VER_MAJOR (work->hdr.version) < 3) {
+    if (work->parse_size < 3)
+      goto not_enough_data;
+
+    mime_str = g_strndup ((gchar *) work->parse_data, 3);
+    len = 3;
+  } else {
+    if (!parse_id_string (work, &mime_str, &len, &datalen))
+      return FALSE;
+    ++len;                      /* for string terminator */
   }
 
-  is_pic_uri = (mime_str != NULL && strcmp (mime_str, "-->") == 0);
-
-  if (mime_str && *mime_str && strchr (mime_str, '/') == NULL && !is_pic_uri) {
-    gchar *tmp;
-
-    tmp = g_strdup_printf ("image/%s", mime_str);
-    g_free (mime_str);
-    mime_str = tmp;
-  }
-
-  if (work->parse_size < (len + 1) + 1 + 1 + 1)
+  if (work->parse_size < len + 1 + 1 + 1)
     goto not_enough_data;
 
-  work->parse_data += (len + 1);
-  work->parse_size -= (len + 1);
+  work->parse_data += len;
+  work->parse_size -= len;
 
+  /* Read image type */
   pic_type = work->parse_data[0];
   ++work->parse_data;
   --work->parse_size;
@@ -490,55 +558,9 @@ parse_picture_frame (ID3TagsWorking * work)
   if (work->parse_size <= 0)
     goto not_enough_data;
 
-  if (is_pic_uri) {
-    gchar *uri;
-
-    uri = g_strndup ((gchar *) work->parse_data, work->parse_size);
-    GST_DEBUG ("image URI: %s", uri);
-
-    image = gst_buffer_new ();
-    GST_BUFFER_MALLOCDATA (image) = (guint8 *) uri;     /* take ownership */
-    GST_BUFFER_DATA (image) = (guint8 *) uri;
-    GST_BUFFER_SIZE (image) = work->parse_size;
-
-    image_caps = gst_caps_new_simple ("text/uri-list", NULL);
-  } else {
-    image = gst_buffer_new_and_alloc (work->parse_size);
-    memcpy (GST_BUFFER_DATA (image), work->parse_data, work->parse_size);
-
-    /* if possible use GStreamer media type rather than declared type */
-    image_caps = gst_type_find_helper_for_buffer (NULL, image, NULL);
-    if (image_caps) {
-      GST_DEBUG ("Found GStreamer media type: %" GST_PTR_FORMAT, image_caps);
-    } else if (mime_str && *mime_str) {
-      GST_DEBUG ("No GStreamer media type found, using declared type");
-      image_caps = gst_caps_new_simple (mime_str, NULL);
-    } else {
-      GST_DEBUG ("Empty declared mime type, ignoring image frame");
-      image = NULL;
-      image_caps = NULL;
-      goto error;
-    }
-  }
-
-  if (image && image_caps) {
-    if (pic_type > 0x14)
-      pic_type = GST_TAG_IMAGE_TYPE_UNDEFINED;
-    gst_structure_set (gst_caps_get_structure (image_caps, 0),
-        "image-type", GST_TYPE_TAG_IMAGE_TYPE,
-        (GstTagImageType) pic_type, NULL);
-
-    gst_buffer_set_caps (image, image_caps);
-    gst_caps_unref (image_caps);
-    if (pic_type == 0x01 || pic_type == 0x02) {
-      /* file icon of some sort */
-      gst_tag_list_add (work->tags, GST_TAG_MERGE_APPEND,
-          GST_TAG_PREVIEW_IMAGE, image, NULL);
-    } else {
-      gst_tag_list_add (work->tags, GST_TAG_MERGE_APPEND,
-          GST_TAG_IMAGE, image, NULL);
-    }
-    gst_buffer_unref (image);
+  if (!gst_tag_list_add_id3_image (work->tags, (guint8 *) work->parse_data,
+          work->parse_size, pic_type)) {
+    goto error;
   }
 
   g_free (mime_str);
@@ -635,6 +657,23 @@ parse_relative_volume_adjustment_two (ID3TagsWorking * work)
   return (gain_tag_name != NULL || peak_tag_name != NULL);
 }
 
+static void
+parse_obsolete_tdat_frame (ID3TagsWorking * work)
+{
+  if (work->parse_size >= 5 &&
+      work->parse_data[0] == ID3V2_ENCODING_ISO8859 &&
+      g_ascii_isdigit (work->parse_data[1]) &&
+      g_ascii_isdigit (work->parse_data[2]) &&
+      g_ascii_isdigit (work->parse_data[3]) &&
+      g_ascii_isdigit (work->parse_data[4])) {
+    work->pending_day = (10 * g_ascii_digit_value (work->parse_data[1])) +
+        g_ascii_digit_value (work->parse_data[2]);
+    work->pending_month = (10 * g_ascii_digit_value (work->parse_data[3])) +
+        g_ascii_digit_value (work->parse_data[4]);
+    GST_LOG ("date (dd/mm) %02u/%02u", work->pending_day, work->pending_month);
+  }
+}
+
 static gboolean
 id3v2_tag_to_taglist (ID3TagsWorking * work, const gchar * tag_name,
     const gchar * tag_str)
@@ -692,14 +731,23 @@ id3v2_tag_to_taglist (ID3TagsWorking * work, const gchar * tag_name,
       break;
     }
     case G_TYPE_STRING:{
-      if (!strcmp (tag_name, GST_TAG_GENRE)) {
-        if (work->prev_genre && !strcmp (tag_str, work->prev_genre))
-          break;                /* Same as the last genre */
-        g_free (work->prev_genre);
-        work->prev_genre = g_strdup (tag_str);
+      const GValue *val;
+      guint i, num;
+
+      /* make sure we add each unique string only once per tag, we don't want
+       * to have the same genre in the genre list multiple times, for example,
+       * or the same DiscID in there twice just because it's contained in the
+       * tag multiple times under different TXXX user tags */
+      num = gst_tag_list_get_tag_size (tag_list, tag_name);
+      for (i = 0; i < num; ++i) {
+        val = gst_tag_list_get_value_index (tag_list, tag_name, i);
+        if (val != NULL && strcmp (g_value_get_string (val), tag_str) == 0)
+          break;
+      }
+      if (i == num) {
+        gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
+            tag_name, tag_str, NULL);
       }
-      gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
-          tag_name, tag_str, NULL);
       break;
     }
 
