@@ -16,6 +16,11 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+/**
+ * SECTION:element-alawenc
+ *
+ * This element encode alaw audio. Alaw coding is also known as G.711.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -26,8 +31,8 @@
 GST_DEBUG_CATEGORY_STATIC (alaw_enc_debug);
 #define GST_CAT_DEFAULT alaw_enc_debug
 
-extern GstPadTemplate *alawenc_src_template;
-extern GstPadTemplate *alawenc_sink_template;
+extern GstStaticPadTemplate alaw_enc_src_factory;
+extern GstStaticPadTemplate alaw_enc_sink_factory;
 
 static GstFlowReturn gst_alaw_enc_chain (GstPad * pad, GstBuffer * buffer);
 
@@ -387,8 +392,10 @@ gst_alaw_enc_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class, alawenc_src_template);
-  gst_element_class_add_pad_template (element_class, alawenc_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&alaw_enc_src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&alaw_enc_sink_factory));
 
   gst_element_class_set_details_simple (element_class,
       "A Law audio encoder", "Codec/Encoder/Audio",
@@ -407,7 +414,8 @@ gst_alaw_enc_class_init (GstALawEncClass * klass)
 static void
 gst_alaw_enc_init (GstALawEnc * alawenc, GstALawEncClass * klass)
 {
-  alawenc->sinkpad = gst_pad_new_from_template (alawenc_sink_template, "sink");
+  alawenc->sinkpad =
+      gst_pad_new_from_static_template (&alaw_enc_sink_factory, "sink");
   gst_pad_set_setcaps_function (alawenc->sinkpad,
       GST_DEBUG_FUNCPTR (gst_alaw_enc_setcaps));
   gst_pad_set_getcaps_function (alawenc->sinkpad,
@@ -416,11 +424,13 @@ gst_alaw_enc_init (GstALawEnc * alawenc, GstALawEncClass * klass)
       GST_DEBUG_FUNCPTR (gst_alaw_enc_chain));
   gst_element_add_pad (GST_ELEMENT (alawenc), alawenc->sinkpad);
 
-  alawenc->srcpad = gst_pad_new_from_template (alawenc_src_template, "src");
+  alawenc->srcpad =
+      gst_pad_new_from_static_template (&alaw_enc_src_factory, "src");
   gst_pad_set_setcaps_function (alawenc->srcpad,
       GST_DEBUG_FUNCPTR (gst_alaw_enc_setcaps));
   gst_pad_set_getcaps_function (alawenc->srcpad,
       GST_DEBUG_FUNCPTR (gst_alaw_enc_getcaps));
+  gst_pad_use_fixed_caps (alawenc->srcpad);
   gst_element_add_pad (GST_ELEMENT (alawenc), alawenc->srcpad);
 
   /* init rest */
@@ -457,11 +467,6 @@ gst_alaw_enc_chain (GstPad * pad, GstBuffer * buffer)
   GST_LOG_OBJECT (alawenc, "buffer with ts=%" GST_TIME_FORMAT,
       GST_TIME_ARGS (timestamp));
 
-  if (duration == GST_CLOCK_TIME_NONE) {
-    duration = gst_util_uint64_scale_int (alaw_size,
-        GST_SECOND, alawenc->rate * alawenc->channels);
-  }
-
   ret =
       gst_pad_alloc_buffer_and_set_caps (alawenc->srcpad,
       GST_BUFFER_OFFSET_NONE, alaw_size, GST_PAD_CAPS (alawenc->srcpad),
@@ -469,6 +474,17 @@ gst_alaw_enc_chain (GstPad * pad, GstBuffer * buffer)
   if (ret != GST_FLOW_OK)
     goto done;
 
+  if (duration == GST_CLOCK_TIME_NONE) {
+    duration = gst_util_uint64_scale_int (alaw_size,
+        GST_SECOND, alawenc->rate * alawenc->channels);
+  }
+
+  if (GST_BUFFER_SIZE (outbuf) < alaw_size) {
+    /* pad-alloc can return a smaller buffer */
+    gst_buffer_unref (outbuf);
+    outbuf = gst_buffer_new_and_alloc (alaw_size);
+  }
+
   alaw_data = (guint8 *) GST_BUFFER_DATA (outbuf);
 
   /* copy discont flag */
