@@ -56,6 +56,7 @@ G_BEGIN_DECLS
 #define FOURCC_stsd     GST_MAKE_FOURCC('s','t','s','d')
 #define FOURCC_stts     GST_MAKE_FOURCC('s','t','t','s')
 #define FOURCC_stss     GST_MAKE_FOURCC('s','t','s','s')
+#define FOURCC_stps     GST_MAKE_FOURCC('s','t','p','s')
 #define FOURCC_stsc     GST_MAKE_FOURCC('s','t','s','c')
 #define FOURCC_stsz     GST_MAKE_FOURCC('s','t','s','z')
 #define FOURCC_stco     GST_MAKE_FOURCC('s','t','c','o')
@@ -87,12 +88,16 @@ G_BEGIN_DECLS
 #define FOURCC__grp     GST_MAKE_FOURCC(0xa9,'g','r','p')
 #define FOURCC__alb     GST_MAKE_FOURCC(0xa9,'a','l','b')
 #define FOURCC__day     GST_MAKE_FOURCC(0xa9,'d','a','y')
+#define FOURCC__req     GST_MAKE_FOURCC(0xa9,'r','e','q')
+#define FOURCC__enc     GST_MAKE_FOURCC(0xa9,'e','n','c')
+#define FOURCC_cprt     GST_MAKE_FOURCC('c','p','r','t')
 #define FOURCC_gnre     GST_MAKE_FOURCC('g','n','r','e')
 #define FOURCC_disc     GST_MAKE_FOURCC('d','i','s','c')
 #define FOURCC_disk     GST_MAKE_FOURCC('d','i','s','k')
 #define FOURCC_trkn     GST_MAKE_FOURCC('t','r','k','n')
 #define FOURCC_cpil     GST_MAKE_FOURCC('c','p','i','l')
 #define FOURCC_tmpo     GST_MAKE_FOURCC('t','m','p','o')
+#define FOURCC_covr     GST_MAKE_FOURCC('c','o','v','r')
 #define FOURCC__too     GST_MAKE_FOURCC(0xa9,'t','o','o')
 #define FOURCC_____     GST_MAKE_FOURCC('-','-','-','-')
 #define FOURCC_free     GST_MAKE_FOURCC('f','r','e','e')
