@@ -42,7 +42,7 @@ GST_DEBUG_CATEGORY_STATIC (rtph264pay_debug);
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_h264pay_details =
-GST_ELEMENT_DETAILS ("RTP packet payloader",
+GST_ELEMENT_DETAILS ("RTP H264 payloader",
     "Codec/Payloader/Network",
     "Payload-encode H264 video into RTP packets (RFC 3984)",
     "Laurent Glayal <spglegle@yahoo.fr>");
@@ -304,6 +304,8 @@ gst_rtp_h264_pay_setcaps (GstBaseRTPPayload * basepayload, GstCaps * caps)
       data += 2;
       size -= 2;
 
+      GST_LOG_OBJECT (rtph264pay, "SPS %d size %d", i, nal_size);
+
       if (size < nal_size)
         goto avcc_error;
 
@@ -335,6 +337,8 @@ gst_rtp_h264_pay_setcaps (GstBaseRTPPayload * basepayload, GstCaps * caps)
       data += 2;
       size -= 2;
 
+      GST_LOG_OBJECT (rtph264pay, "PPS %d size %d", i, nal_size);
+
       if (size < nal_size)
         goto avcc_error;
 
