@@ -1,6 +1,6 @@
 /* GStreamer
  * Copyright (C) <2007> Nokia Corporation (contact <stefan.kost@nokia.com>)
- *               <2007> Wim Taymans <wim@fluendo.com>
+ *               <2007> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -31,22 +31,11 @@ GST_DEBUG_CATEGORY_STATIC (rtpmp4adepay_debug);
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_mp4adepay_details =
-GST_ELEMENT_DETAILS ("RTP packet parser",
+GST_ELEMENT_DETAILS ("RTP MPEG4 audio depayloader",
     "Codec/Depayloader/Network",
     "Extracts MPEG4 audio from RTP packets (RFC 3016)",
     "Nokia Corporation (contact <stefan.kost@nokia.com>), "
-    "Wim Taymans <wim@fluendo.com>");
-
-/* RtpMP4ADepay signals and args */
-enum
-{
-  LAST_SIGNAL
-};
-
-enum
-{
-  PROP_0,
-};
+    "Wim Taymans <wim.taymans@gmail.com>");
 
 static GstStaticPadTemplate gst_rtp_mp4a_depay_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
@@ -62,6 +51,7 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"audio\", "
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
         "clock-rate = (int) [1, MAX ], "
         "encoding-name = (string) \"MP4A-LATM\""
         /* All optional parameters
@@ -82,11 +72,6 @@ static gboolean gst_rtp_mp4a_depay_setcaps (GstBaseRTPDepayload * depayload,
 static GstBuffer *gst_rtp_mp4a_depay_process (GstBaseRTPDepayload * depayload,
     GstBuffer * buf);
 
-static void gst_rtp_mp4a_depay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec);
-static void gst_rtp_mp4a_depay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec);
-
 static GstStateChangeReturn gst_rtp_mp4a_depay_change_state (GstElement *
     element, GstStateChange transition);
 
@@ -116,8 +101,6 @@ gst_rtp_mp4a_depay_class_init (GstRtpMP4ADepayClass * klass)
   gstbasertpdepayload_class = (GstBaseRTPDepayloadClass *) klass;
 
   gobject_class->finalize = gst_rtp_mp4a_depay_finalize;
-  gobject_class->set_property = gst_rtp_mp4a_depay_set_property;
-  gobject_class->get_property = gst_rtp_mp4a_depay_get_property;
 
   gstelement_class->change_state = gst_rtp_mp4a_depay_change_state;
 
@@ -155,20 +138,21 @@ gst_rtp_mp4a_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   GstRtpMP4ADepay *rtpmp4adepay;
   GstCaps *srccaps;
   const gchar *str;
-  gint clock_rate = 90000;      /* default */
-  gint object_type = 2;         /* AAC LC default */
+  gint clock_rate;
+  gint object_type;
   gint channels = 2;            /* default */
+  gboolean res;
 
   rtpmp4adepay = GST_RTP_MP4A_DEPAY (depayload);
 
   structure = gst_caps_get_structure (caps, 0);
 
-  if (gst_structure_has_field (structure, "clock-rate"))
-    gst_structure_get_int (structure, "clock-rate", &clock_rate);
+  if (!gst_structure_get_int (structure, "clock-rate", &clock_rate))
+    clock_rate = 90000;         /* default */
   depayload->clock_rate = clock_rate;
 
-  if (gst_structure_has_field (structure, "object"))
-    gst_structure_get_int (structure, "object", &object_type);
+  if (!gst_structure_get_int (structure, "object", &object_type))
+    object_type = 2;            /* AAC LC default */
 
   srccaps = gst_caps_new_simple ("audio/mpeg",
       "mpegversion", G_TYPE_INT, 4,
@@ -190,6 +174,7 @@ gst_rtp_mp4a_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
 
       data = GST_BUFFER_DATA (buffer);
       size = GST_BUFFER_SIZE (buffer);
+
       if (size < 2) {
         GST_WARNING_OBJECT (depayload, "config too short (%d < 2)", size);
         goto bad_config;
@@ -199,7 +184,7 @@ gst_rtp_mp4a_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
        *
        * audioMuxVersion           == 0 (1 bit)
        * allStreamsSameTimeFraming == 1 (1 bit)
-       * numSubFrames              == 0 (6 bits)
+       * numSubFrames              == rtpmp4adepay->numSubFrames (6 bits)
        * numProgram                == 0 (4 bits)
        * numLayer                  == 0 (3 bits)
        *
@@ -213,6 +198,11 @@ gst_rtp_mp4a_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
         goto bad_config;
       }
 
+      rtpmp4adepay->numSubFrames = (data[0] & 0x3F);
+
+      GST_LOG_OBJECT (rtpmp4adepay, "numSubFrames %d",
+          rtpmp4adepay->numSubFrames);
+
       /* shift rest of string 15 bits down */
       size -= 2;
       for (i = 0; i < size; i++) {
@@ -230,10 +220,10 @@ gst_rtp_mp4a_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
     }
   }
 bad_config:
-  gst_pad_set_caps (depayload->srcpad, srccaps);
+  res = gst_pad_set_caps (depayload->srcpad, srccaps);
   gst_caps_unref (srccaps);
 
-  return TRUE;
+  return res;
 }
 
 static GstBuffer *
@@ -244,9 +234,6 @@ gst_rtp_mp4a_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 
   rtpmp4adepay = GST_RTP_MP4A_DEPAY (depayload);
 
-  if (!gst_rtp_buffer_validate (buf))
-    goto bad_packet;
-
   /* flush remaining data on discont */
   if (GST_BUFFER_IS_DISCONT (buf)) {
     gst_adapter_clear (rtpmp4adepay->adapter);
@@ -260,79 +247,87 @@ gst_rtp_mp4a_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
    * and push a buffer */
   if (gst_rtp_buffer_get_marker (buf)) {
     guint avail;
-    guint latm_header_len;
-    guint data_len;
+    guint i;
     guint8 *data;
+    guint pos;
 
     avail = gst_adapter_available (rtpmp4adepay->adapter);
 
+    GST_LOG_OBJECT (rtpmp4adepay, "have marker and %u available", avail);
+
     outbuf = gst_adapter_take_buffer (rtpmp4adepay->adapter, avail);
-
-    /* determine payload length and set buffer data pointer accordingly */
-    /* FIXME, check for overrun */
-    latm_header_len = 0;
-    data_len = 0;
     data = GST_BUFFER_DATA (outbuf);
-    do {
-      data_len += data[latm_header_len];
-    } while (data[latm_header_len++] == 0xff);
+    /* position in data we are at */
+    pos = 0;
 
-    /* just a check that lengths match, possibly there can be more than one
-     * audioMuxElement in the payload? */
-    if ((data_len + latm_header_len) != avail) {
-      GST_WARNING_OBJECT (depayload, "not all payload consumed");
+    /* looping through the number of sub-frames in the audio payload */
+    for (i = 0; i <= rtpmp4adepay->numSubFrames; i++) {
+      /* determine payload length and set buffer data pointer accordingly */
+      guint skip;
+      guint data_len;
+      guint32 timestamp;
+
+      GstBuffer *tmp = NULL;
+
+      timestamp = gst_rtp_buffer_get_timestamp (buf);
+
+      /* each subframe starts with a variable length encoding */
+      data_len = 0;
+      for (skip = 0; skip < avail; skip++) {
+        data_len += data[skip];
+        if (data[skip] != 0xff)
+          break;
+      }
+      skip++;
+
+      /* this can not be possible, we have not enough data or the length
+       * decoding failed because we ran out of data. */
+      if (skip + data_len > avail)
+        goto wrong_size;
+
+      GST_LOG_OBJECT (rtpmp4adepay,
+          "subframe %u, header len %u, data len %u, left %u", i, skip, data_len,
+          avail);
+
+      /* take data out, skip the header */
+      pos += skip;
+      tmp = gst_buffer_create_sub (outbuf, pos, data_len);
+
+      /* skip data too */
+      skip += data_len;
+      pos += data_len;
+
+      /* update our pointers whith what we consumed */
+      data += skip;
+      avail -= skip;
+
+      gst_buffer_set_caps (tmp, GST_PAD_CAPS (depayload->srcpad));
+
+      /* only apply the timestamp for the first buffer. Based on gstrtpmp4gdepay.c */
+      if (i == 0)
+        gst_base_rtp_depayload_push_ts (depayload, timestamp, tmp);
+      else
+        gst_base_rtp_depayload_push (depayload, tmp);
     }
 
-    GST_BUFFER_SIZE (outbuf) = avail - latm_header_len;
-    GST_BUFFER_DATA (outbuf) += latm_header_len;
-
-    gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
-
-    GST_DEBUG ("gst_rtp_mp4a_depay_process: pushing buffer of size %d",
-        GST_BUFFER_SIZE (outbuf));
-
-    return outbuf;
+    /* just a check that lengths match */
+    if (avail) {
+      GST_ELEMENT_WARNING (depayload, STREAM, DECODE,
+          ("Packet invalid"), ("Not all payload consumed: "
+              "possible wrongly encoded packet."));
+    }
   }
   return NULL;
 
-bad_packet:
+  /* ERRORS */
+wrong_size:
   {
     GST_ELEMENT_WARNING (rtpmp4adepay, STREAM, DECODE,
-        ("Packet did not validate"), (NULL));
+        ("Packet did not validate"), ("wrong packet size"));
     return NULL;
   }
 }
 
-static void
-gst_rtp_mp4a_depay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
-{
-  GstRtpMP4ADepay *rtpmp4adepay;
-
-  rtpmp4adepay = GST_RTP_MP4A_DEPAY (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static void
-gst_rtp_mp4a_depay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec)
-{
-  GstRtpMP4ADepay *rtpmp4adepay;
-
-  rtpmp4adepay = GST_RTP_MP4A_DEPAY (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
 static GstStateChangeReturn
 gst_rtp_mp4a_depay_change_state (GstElement * element,
     GstStateChange transition)
