@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2005> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2005> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -31,21 +31,10 @@ GST_DEBUG_CATEGORY_STATIC (rtpmp4vdepay_debug);
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_mp4vdepay_details =
-GST_ELEMENT_DETAILS ("RTP packet depayloader",
+GST_ELEMENT_DETAILS ("RTP MPEG4 video depayloader",
     "Codec/Depayloader/Network",
     "Extracts MPEG4 video from RTP packets (RFC 3016)",
-    "Wim Taymans <wim@fluendo.com>");
-
-/* RtpMP4VDepay signals and args */
-enum
-{
-  LAST_SIGNAL
-};
-
-enum
-{
-  PROP_0,
-};
+    "Wim Taymans <wim.taymans@gmail.com>");
 
 static GstStaticPadTemplate gst_rtp_mp4v_depay_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
@@ -81,11 +70,6 @@ static gboolean gst_rtp_mp4v_depay_setcaps (GstBaseRTPDepayload * depayload,
 static GstBuffer *gst_rtp_mp4v_depay_process (GstBaseRTPDepayload * depayload,
     GstBuffer * buf);
 
-static void gst_rtp_mp4v_depay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec);
-static void gst_rtp_mp4v_depay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec);
-
 static GstStateChangeReturn gst_rtp_mp4v_depay_change_state (GstElement *
     element, GstStateChange transition);
 
@@ -115,8 +99,6 @@ gst_rtp_mp4v_depay_class_init (GstRtpMP4VDepayClass * klass)
   gstbasertpdepayload_class = (GstBaseRTPDepayloadClass *) klass;
 
   gobject_class->finalize = gst_rtp_mp4v_depay_finalize;
-  gobject_class->set_property = gst_rtp_mp4v_depay_set_property;
-  gobject_class->get_property = gst_rtp_mp4v_depay_get_property;
 
   gstelement_class->change_state = gst_rtp_mp4v_depay_change_state;
 
@@ -154,14 +136,15 @@ gst_rtp_mp4v_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   GstRtpMP4VDepay *rtpmp4vdepay;
   GstCaps *srccaps;
   const gchar *str;
-  gint clock_rate = 90000;      /* default */
+  gint clock_rate;
+  gboolean res;
 
   rtpmp4vdepay = GST_RTP_MP4V_DEPAY (depayload);
 
   structure = gst_caps_get_structure (caps, 0);
 
-  if (gst_structure_has_field (structure, "clock-rate"))
-    gst_structure_get_int (structure, "clock-rate", &clock_rate);
+  if (!gst_structure_get_int (structure, "clock-rate", &clock_rate))
+    clock_rate = 90000;         /* default */
   depayload->clock_rate = clock_rate;
 
   srccaps = gst_caps_new_simple ("video/mpeg",
@@ -185,10 +168,10 @@ gst_rtp_mp4v_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
       g_warning ("cannot convert config to buffer");
     }
   }
-  gst_pad_set_caps (depayload->srcpad, srccaps);
+  res = gst_pad_set_caps (depayload->srcpad, srccaps);
   gst_caps_unref (srccaps);
 
-  return TRUE;
+  return res;
 }
 
 static GstBuffer *
@@ -199,9 +182,6 @@ gst_rtp_mp4v_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 
   rtpmp4vdepay = GST_RTP_MP4V_DEPAY (depayload);
 
-  if (!gst_rtp_buffer_validate (buf))
-    goto bad_packet;
-
   /* flush remaining data on discont */
   if (GST_BUFFER_IS_DISCONT (buf))
     gst_adapter_clear (rtpmp4vdepay->adapter);
@@ -216,7 +196,6 @@ gst_rtp_mp4v_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     avail = gst_adapter_available (rtpmp4vdepay->adapter);
 
     outbuf = gst_adapter_take_buffer (rtpmp4vdepay->adapter, avail);
-    gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
 
     GST_DEBUG ("gst_rtp_mp4v_depay_chain: pushing buffer of size %d",
         GST_BUFFER_SIZE (outbuf));
@@ -224,43 +203,6 @@ gst_rtp_mp4v_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     return outbuf;
   }
   return NULL;
-
-bad_packet:
-  {
-    GST_ELEMENT_WARNING (rtpmp4vdepay, STREAM, DECODE,
-        ("Packet did not validate"), (NULL));
-    return NULL;
-  }
-}
-
-static void
-gst_rtp_mp4v_depay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
-{
-  GstRtpMP4VDepay *rtpmp4vdepay;
-
-  rtpmp4vdepay = GST_RTP_MP4V_DEPAY (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static void
-gst_rtp_mp4v_depay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec)
-{
-  GstRtpMP4VDepay *rtpmp4vdepay;
-
-  rtpmp4vdepay = GST_RTP_MP4V_DEPAY (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
 }
 
 static GstStateChangeReturn
