@@ -31,7 +31,7 @@
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_pcma_pay_details =
-GST_ELEMENT_DETAILS ("RTP packet payloader",
+GST_ELEMENT_DETAILS ("RTP PCMA payloader",
     "Codec/Payloader/Network",
     "Payload-encodes PCMA audio into a RTP packet",
     "Edgard Lima <edgard.lima@indt.org.br>");
@@ -110,12 +110,14 @@ gst_rtp_pcma_pay_init (GstRtpPmcaPay * rtppcmapay, GstRtpPmcaPayClass * klass)
 static gboolean
 gst_rtp_pcma_pay_setcaps (GstBaseRTPPayload * payload, GstCaps * caps)
 {
+  gboolean res;
+
   payload->pt = GST_RTP_PAYLOAD_PCMA;
+
   gst_basertppayload_set_options (payload, "audio", FALSE, "PCMA", 8000);
+  res = gst_basertppayload_set_outcaps (payload, NULL);
 
-  gst_basertppayload_set_outcaps (payload, NULL);
-
-  return TRUE;
+  return res;
 }
 
 gboolean
