@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2005> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2005> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -28,10 +28,10 @@
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_sv3vdepay_details =
-GST_ELEMENT_DETAILS ("RTP packet depayloader",
+GST_ELEMENT_DETAILS ("RTP SVQ3 depayloader",
     "Codec/Depayloader/Network",
     "Extracts SVQ3 video from RTP packets (no RFC)",
-    "Wim Taymans <wim@fluendo.com>");
+    "Wim Taymans <wim.taymans@gmail.com>");
 
 static GstStaticPadTemplate gst_rtp_sv3v_depay_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
@@ -123,32 +123,27 @@ gst_rtp_sv3v_depay_finalize (GObject * object)
 gboolean
 gst_rtp_sv3v_depay_setcaps (GstBaseRTPDepayload * filter, GstCaps * caps)
 {
-
   GstStructure *structure = gst_caps_get_structure (caps, 0);
-  gint clock_rate = 90000;      // default
-
-  if (gst_structure_has_field (structure, "clock-rate"))
-    gst_structure_get_int (structure, "clock-rate", &clock_rate);
+  gint clock_rate;
 
+  if (!gst_structure_get_int (structure, "clock-rate", &clock_rate))
+    clock_rate = 90000;         // default
   filter->clock_rate = clock_rate;
 
+  /* will set caps later */
+
   return TRUE;
 }
 
-
 static GstBuffer *
 gst_rtp_sv3v_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 {
-
   GstRtpSV3VDepay *rtpsv3vdepay;
   GstBuffer *outbuf;
   guint16 seq;
 
   rtpsv3vdepay = GST_RTP_SV3V_DEPAY (depayload);
 
-  if (!gst_rtp_buffer_validate (buf))
-    goto bad_packet;
-
   /* flush on sequence number gaps */
   seq = gst_rtp_buffer_get_seq (buf);
   if (seq != rtpsv3vdepay->nextseq) {
@@ -236,9 +231,6 @@ gst_rtp_sv3v_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
       avail = gst_adapter_available (rtpsv3vdepay->adapter);
       outbuf = gst_adapter_take_buffer (rtpsv3vdepay->adapter, avail);
 
-      /* timestamp for complete buffer is that of last buffer as well */
-      gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
-
       return outbuf;
     }
   }
@@ -248,7 +240,7 @@ gst_rtp_sv3v_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 bad_packet:
   {
     GST_ELEMENT_WARNING (rtpsv3vdepay, STREAM, DECODE,
-        ("Packet did not validate"), (NULL));
+        (NULL), ("Packet was too short"));
     return NULL;
   }
 }
