@@ -21,13 +21,9 @@
  * SECTION:element-multiupdsink
  * @see_also: udpsink, multifdsink
  *
- * <refsect2>
- * <para>
  * multiudpsink is a network sink that sends UDP packets to multiple
  * clients.
  * It can be combined with rtp payload encoders to implement RTP streaming.
- * </para>
- * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -105,9 +101,14 @@ enum
 };
 
 #define CLOSE_IF_REQUESTED(udpctx)                                        \
-  if ((!udpctx->externalfd) || (udpctx->externalfd && udpctx->closefd))   \
+G_STMT_START {                                                            \
+  if ((!udpctx->externalfd) || (udpctx->externalfd && udpctx->closefd)) { \
     CLOSE_SOCKET(udpctx->sock);                                           \
-  udpctx->sock = -1;
+    if (udpctx->sock == udpctx->sockfd)                                   \
+      udpctx->sockfd = DEFAULT_SOCKFD;                                    \
+  }                                                                       \
+  udpctx->sock = DEFAULT_SOCK;                                            \
+} G_STMT_END
 
 static void gst_multiudpsink_base_init (gpointer g_class);
 static void gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass);
@@ -332,7 +333,7 @@ gst_multiudpsink_init (GstMultiUDPSink * sink)
   WSA_STARTUP (sink);
 
   sink->client_lock = g_mutex_new ();
-  sink->sock = -1;
+  sink->sock = DEFAULT_SOCK;
   sink->sockfd = DEFAULT_SOCKFD;
   sink->closefd = DEFAULT_CLOSEFD;
   sink->externalfd = (sink->sockfd != -1);
@@ -352,6 +353,9 @@ gst_multiudpsink_finalize (GObject * object)
   g_list_foreach (sink->clients, (GFunc) free_client, NULL);
   g_list_free (sink->clients);
 
+  if (sink->sockfd >= 0 && sink->closefd)
+    CLOSE_SOCKET (sink->sockfd);
+
   g_mutex_free (sink->client_lock);
 
   WSA_CLEANUP (object);
@@ -363,9 +367,10 @@ static GstFlowReturn
 gst_multiudpsink_render (GstBaseSink * bsink, GstBuffer * buffer)
 {
   GstMultiUDPSink *sink;
-  gint ret, size, num = 0;
+  gint ret, size, num = 0, no_clients = 0;
   guint8 *data;
   GList *clients;
+  gint len;
 
   sink = GST_MULTIUDPSINK (bsink);
 
@@ -383,22 +388,30 @@ gst_multiudpsink_render (GstBaseSink * bsink, GstBuffer * buffer)
     GstUDPClient *client;
 
     client = (GstUDPClient *) clients->data;
-    num++;
+    no_clients++;
     GST_LOG_OBJECT (sink, "sending %d bytes to client %p", size, client);
 
     while (TRUE) {
-      ret = sendto (*client->sock, data, size, 0,
-          (struct sockaddr *) &client->theiraddr, sizeof (client->theiraddr));
+      len = gst_udp_get_sockaddr_length (&client->theiraddr);
+
+      ret = sendto (*client->sock,
+#ifdef G_OS_WIN32
+          (char *) data,
+#else
+          data,
+#endif
+          size, 0, (struct sockaddr *) &client->theiraddr, len);
 
       if (ret < 0) {
-        /* we get a non-posix EPERM on Linux when a firewall rule blocks this
-         * destination. We will simply ignore this. */
-        if (errno == EPERM)
-          break;
+        /* some error, just warn, it's likely recoverable and we don't want to
+         * break streaming. We break so that we stop retrying for this client. */
         if (errno != EINTR && errno != EAGAIN) {
-          goto send_error;
+          GST_WARNING_OBJECT (sink, "client %p gave error %d (%s)", client,
+              errno, g_strerror (errno));
+          break;
         }
       } else {
+        num++;
         client->bytes_sent += ret;
         client->packets_sent++;
         sink->bytes_served += ret;
@@ -408,20 +421,10 @@ gst_multiudpsink_render (GstBaseSink * bsink, GstBuffer * buffer)
   }
   g_mutex_unlock (sink->client_lock);
 
-  GST_LOG_OBJECT (sink, "sent %d bytes to %d clients", size, num);
+  GST_LOG_OBJECT (sink, "sent %d bytes to %d (of %d) clients", size, num,
+      no_clients);
 
   return GST_FLOW_OK;
-
-  /* ERRORS */
-send_error:
-  {
-    /* if sendto returns an error, something is seriously wrong */
-    g_mutex_unlock (sink->client_lock);
-    GST_DEBUG_OBJECT (sink, "got send error %d: %s", errno, g_strerror (errno));
-    GST_ELEMENT_ERROR (sink, STREAM, FAILED, (NULL),
-        ("Got send error %d: %s", errno, g_strerror (errno)));
-    return GST_FLOW_ERROR;
-  }
 }
 
 static void
@@ -517,6 +520,9 @@ gst_multiudpsink_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_SOCKFD:
+      if (udpsink->sockfd >= 0 && udpsink->sockfd != udpsink->sock &&
+          udpsink->closefd)
+        CLOSE_SOCKET (udpsink->sockfd);
       udpsink->sockfd = g_value_get_int (value);
       GST_DEBUG_OBJECT (udpsink, "setting SOCKFD to %d", udpsink->sockfd);
       break;
@@ -536,7 +542,7 @@ gst_multiudpsink_set_property (GObject * object, guint prop_id,
       udpsink->loop = g_value_get_boolean (value);
       break;
     case PROP_QOS_DSCP:
-      udpsink->qos_dscp = g_value_get_uint (value);
+      udpsink->qos_dscp = g_value_get_int (value);
       gst_multiudpsink_setup_qos_dscp (udpsink);
       break;
     default:
