@@ -16,12 +16,33 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
+/**
+ * SECTION:element-udpsink
+ * @see_also: udpsrc, multifdsink
+ *
+ * udpsink is a network sink that sends UDP packets to the network.
+ * It can be combined with RTP payloaders to implement RTP streaming.
+ *
+ * <refsect2>
+ * <title>Examples</title>
+ * |[
+ * gst-launch -v audiotestsrc ! udpsink
+ * ]|
+ * </refsect2>
+ */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 #include "gstudpsink.h"
 
+#include <stdio.h>
+#include <stdlib.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#include <errno.h>
+#include <string.h>
+
 #define UDP_DEFAULT_HOST        "localhost"
 #define UDP_DEFAULT_PORT        4951
 
@@ -149,6 +170,7 @@ static void
 gst_udpsink_finalize (GstUDPSink * udpsink)
 {
   g_free (udpsink->host);
+  g_free (udpsink->uri);
 
   G_OBJECT_CLASS (parent_class)->finalize ((GObject *) udpsink);
 }
