@@ -19,33 +19,26 @@
 
 /**
  * SECTION:element-videomixer
- * @short_description: Takes several AYUV video streams as input and mixes them
- * together.
  *
- * <refsect2>
- * <para>
  * Videomixer can only accept AYUV video streams. For each of the requested
  * sink pads it will compare the incoming geometry and framerate to define the
  * output parameters. Indeed output video frames will have the geometry of the
  * biggest incoming video stream and the framerate of the fastest incoming one.
- * </para>
- * <para>
+ *
  * Individual parameters for each input stream can be configured on the
  * #GstVideoMixerPad.
- * </para>
+ *
+ * <refsect2>
  * <title>Sample pipelines</title>
- * <para>
- * Here is a pipeline to demonstrate videomixer used together with videobox :
- * <programlisting>
+ * |[
  * gst-launch videotestsrc pattern=1 ! video/x-raw-yuv, framerate=\(fraction\)10/1, width=100, height=100 ! videobox border-alpha=0 alpha=0.5 top=-70 bottom=-70 right=-220 ! videomixer name=mix ! ffmpegcolorspace ! xvimagesink videotestsrc ! video/x-raw-yuv, framerate=\(fraction\)5/1, width=320, height=240 ! alpha alpha=0.7 ! mix.
- * </programlisting>
+ * ]| A pipeline to demonstrate videomixer used together with videobox.
  * This should show a 320x240 pixels video test source with some transparency
  * showing the background checker pattern. Another video test source with just
  * the snow pattern of 100x100 pixels is overlayed on top of the first one on
  * the left vertically centered with a small transparency showing the first
  * video test source behind and the checker pattern under it. Note that the
  * framerate of the output video is 10 frames per second.
- * </para>
  * </refsect2>
  */
 
@@ -1277,6 +1270,7 @@ gst_videomixer_fill_queues (GstVideoMixer * mix)
     }
     if (mix->sendseg && (mixpad == mix->master)) {
       GstEvent *event;
+      gint64 stop, start;
 
       GstSegment *segment = &data->segment;
 
@@ -1294,8 +1288,18 @@ gst_videomixer_fill_queues (GstVideoMixer * mix)
        * match.
        */
       GST_INFO ("_sending play segment");
+
+      start = segment->accum;
+
+      /* get the duration of the segment if we can and add it to the accumulated
+       * time on the segment. */
+      if (segment->stop != -1 && segment->start != -1)
+        stop = start + (segment->stop - segment->start);
+      else
+        stop = -1;
+
       event = gst_event_new_new_segment_full (FALSE, segment->rate, 1.0,
-          segment->format, 0, -1, mix->segment_position);
+          segment->format, start, stop, start + mix->segment_position);
       gst_pad_push_event (mix->srcpad, event);
       mix->sendseg = FALSE;
     }
@@ -1627,6 +1631,9 @@ gst_videomixer_sink_event (GstPad * pad, GstEvent * event)
        */
       videomixer->sendseg = TRUE;
       break;
+    case GST_EVENT_NEWSEGMENT:
+      videomixer->sendseg = TRUE;
+      break;
     default:
       break;
   }
