@@ -22,28 +22,20 @@
 /**
  * SECTION:element-wavparse
  *
- * <refsect2>
- * <para>
  * Parse a .wav file into raw or compressed audio.
- * </para>
- * <para>
+ *
  * Wavparse supports both push and pull mode operations, making it possible to
  * stream from a network source.
- * </para>
+ *
+ * <refsect2>
  * <title>Example launch line</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch filesrc location=sine.wav ! wavparse ! audioconvert ! alsasink
- * </programlisting>
- * Read a wav file and output to the soundcard using the ALSA element. The
+ * ]| Read a wav file and output to the soundcard using the ALSA element. The
  * wav file is assumed to contain raw uncompressed samples.
- * </para>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch gnomevfssrc location=http://www.example.org/sine.wav ! queue ! wavparse ! audioconvert ! alsasink
- * </programlisting>
- * Stream data from a network url.
- * </para>
+ * ]| Stream data from a network url.
  * </refsect2>
  *
  * Last reviewed on 2007-02-14 (0.10.6)
@@ -168,6 +160,7 @@ gst_wavparse_reset (GstWavParse * wav)
   wav->seek_event = NULL;
   if (wav->adapter) {
     gst_adapter_clear (wav->adapter);
+    g_object_unref (wav->adapter);
     wav->adapter = NULL;
   }
   if (wav->tags)
@@ -248,7 +241,7 @@ gst_wavparse_create_sourcepad (GstWavParse * wavparse)
 static guint64
 uint64_ceiling_scale_int (guint64 val, gint num, gint denom)
 {
-  guint64 result = gst_util_uint64_scale (val, num, denom);
+  guint64 result = gst_util_uint64_scale_int (val, num, denom);
 
   if (uint64_scale_modulo (val, num, denom) == 0)
     return result;
@@ -260,7 +253,7 @@ uint64_ceiling_scale_int (guint64 val, gint num, gint denom)
 static guint64
 uint64_ceiling_scale (guint64 val, guint64 num, guint64 denom)
 {
-  guint64 result = gst_util_uint64_scale_int (val, num, denom);
+  guint64 result = gst_util_uint64_scale (val, num, denom);
 
   if (uint64_scale_modulo (val, num, denom) == 0)
     return result;
@@ -1046,6 +1039,7 @@ gst_wavparse_calculate_duration (GstWavParse * wav)
     return TRUE;
 
   if (wav->bps > 0) {
+    GST_INFO_OBJECT (wav, "Got datasize %" G_GUINT64_FORMAT, wav->datasize);
     wav->duration =
         uint64_ceiling_scale (wav->datasize, GST_SECOND, (guint64) wav->bps);
     GST_INFO_OBJECT (wav, "Got duration (bps) %" GST_TIME_FORMAT,
@@ -1371,7 +1365,7 @@ gst_wavparse_stream_headers (GstWavParse * wav)
         }
         switch (ltag) {
           case GST_RIFF_LIST_INFO:{
-            const guint data_size = size - 12;
+            const guint data_size = size - 4;
             GstTagList *new;
 
             GST_INFO_OBJECT (wav, "Have LIST chunk INFO");
@@ -2218,12 +2212,17 @@ gst_wavparse_sink_activate (GstPad * sinkpad)
   if (gst_pad_check_pull_range (sinkpad)) {
     GST_DEBUG ("going to pull mode");
     wav->streaming = FALSE;
+    if (wav->adapter) {
+      gst_adapter_clear (wav->adapter);
+      g_object_unref (wav->adapter);
+    }
     wav->adapter = NULL;
     res = gst_pad_activate_pull (sinkpad, TRUE);
   } else {
     GST_DEBUG ("going to push (streaming) mode");
     wav->streaming = TRUE;
-    wav->adapter = gst_adapter_new ();
+    if (wav->adapter == NULL)
+      wav->adapter = gst_adapter_new ();
     res = gst_pad_activate_push (sinkpad, TRUE);
   }
   gst_object_unref (wav);
