@@ -25,21 +25,17 @@
 /**
  * SECTION:element-directdrawsink
  *
- * <refsect2>
- * <para>
  * DirectdrawSink renders video RGB frames to any win32 window. This element
- * can receive a window ID from the application through the XOverlay interface
+ * can receive a window ID from the application through the #XOverlay interface
  * and will then render video frames in this window.
  * If no Window ID was provided by the application, the element will create its
  * own internal window and render into it.
- * </para>
- * <title>Examples</title>
- * <para>
- * Here is a simple pipeline to test the sink :
- * <programlisting>
- * gst-launch-0.10 -v videotestsrc ! directdrawsink
- * </programlisting>
- * </para>
+ *
+ * <refsect2>
+ * <title>Example pipelines</title>
+ * |[
+ * gst-launch -v videotestsrc ! directdrawsink
+ * ]| a simple pipeline to test the sink
  * </refsect2>
  */
 
@@ -614,6 +610,7 @@ gst_directdraw_sink_buffer_alloc (GstBaseSink * bsink, guint64 offset,
               !gst_directdraw_sink_surface_check (ddrawsink, surface))
           ) {
         gst_directdraw_sink_surface_destroy (ddrawsink, surface);
+        gst_buffer_unref (surface);
         surface = NULL;
       } else {
         /* We found a suitable surface */
@@ -800,12 +797,10 @@ gst_directdraw_sink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
 
   if (buf) {
     /* save a reference to the input buffer */
-    if (ddrawsink->last_buffer != buf) {
-      if (ddrawsink->last_buffer) {
-        gst_buffer_unref (ddrawsink->last_buffer);
-      }
-    }
-    ddrawsink->last_buffer = gst_buffer_ref (buf);
+    gst_buffer_ref (buf);
+    if (ddrawsink->last_buffer != NULL)
+      gst_buffer_unref (ddrawsink->last_buffer);
+    ddrawsink->last_buffer = buf;
   } else {
     /* use last buffer */
     buf = ddrawsink->last_buffer;
@@ -1899,6 +1894,7 @@ gst_directdraw_sink_bufferpool_clear (GstDirectDrawSink * ddrawsink)
     ddrawsink->buffer_pool = g_slist_delete_link (ddrawsink->buffer_pool,
         ddrawsink->buffer_pool);
     gst_directdraw_sink_surface_destroy (ddrawsink, surface);
+    gst_buffer_unref (surface);
   }
   g_mutex_unlock (ddrawsink->pool_lock);
 }
