@@ -24,23 +24,19 @@
 /**
  * SECTION:element-v4l2src
  *
+ * v4l2src can be used to capture video from v4l2 devices, like webcams and tv
+ * cards.
+ *
  * <refsect2>
- * v4l2src can be used to capture video from v4l2 devices, like webcams and tv cards.
  * <title>Example launch lines</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch v4l2src ! xvimagesink
- * </programlisting>
- * This pipeline shows the video captured from /dev/video0 tv card and for
+ * ]| This pipeline shows the video captured from /dev/video0 tv card and for
  * webcams.
- * </para>
- * <para>
- * <programlisting>
- * gst-launch-0.10 v4l2src ! jpegdec ! xvimagesink
- * </programlisting>
- * This pipeline shows the video captured from a webcam that delivers jpeg
+ * |[
+ * gst-launch v4l2src ! jpegdec ! xvimagesink
+ * ]| This pipeline shows the video captured from a webcam that delivers jpeg
  * images.
- * </para>
  * </refsect2>
  */
 
@@ -354,6 +350,11 @@ gst_v4l2src_dispose (GObject * object)
     gst_v4l2src_clear_format_list (v4l2src);
   }
 
+  if (v4l2src->probed_caps) {
+    gst_caps_unref (v4l2src->probed_caps);
+  }
+
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -581,15 +582,6 @@ gst_v4l2src_v4l2fourcc_to_structure (guint32 fourcc)
 {
   GstStructure *structure = NULL;
 
-  /* FIXME: new FourCCs
-     camera: ZC0301 PC Camera
-     driver: zc0301
-     BA81, S910, PWC1, PWC2
-
-     camera:
-     driver:
-   */
-
   switch (fourcc) {
     case V4L2_PIX_FMT_MJPEG:   /* Motion-JPEG */
     case V4L2_PIX_FMT_JPEG:    /* JFIF JPEG */
@@ -679,6 +671,9 @@ gst_v4l2src_v4l2fourcc_to_structure (guint32 fourcc)
       break;
     }
     case V4L2_PIX_FMT_GREY:    /*  8  Greyscale     */
+      structure = gst_structure_new ("video/x-raw-gray",
+          "bpp", G_TYPE_INT, 8, NULL);
+      break;
     case V4L2_PIX_FMT_YYUV:    /* 16  YUV 4:2:2     */
     case V4L2_PIX_FMT_HI240:   /*  8  8-bit color   */
       /* FIXME: get correct fourccs here */
@@ -748,9 +743,26 @@ gst_v4l2src_v4l2fourcc_to_structure (guint32 fourcc)
       break;
     case V4L2_PIX_FMT_WNVA:    /* Winnov hw compres */
       break;
+#ifdef V4L2_PIX_FMT_SBGGR8
     case V4L2_PIX_FMT_SBGGR8:
       structure = gst_structure_new ("video/x-raw-bayer", NULL);
       break;
+#endif
+#ifdef V4L2_PIX_FMT_SN9C10X
+    case V4L2_PIX_FMT_SN9C10X:
+      structure = gst_structure_new ("video/x-sonix", NULL);
+      break;
+#endif
+#ifdef V4L2_PIX_FMT_PWC1
+    case V4L2_PIX_FMT_PWC1:
+      structure = gst_structure_new ("video/x-pwc1", NULL);
+      break;
+#endif
+#ifdef V4L2_PIX_FMT_PWC2
+    case V4L2_PIX_FMT_PWC2:
+      structure = gst_structure_new ("video/x-pwc2", NULL);
+      break;
+#endif
     default:
       GST_DEBUG ("Unknown fourcc 0x%08x %" GST_FOURCC_FORMAT,
           fourcc, GST_FOURCC_ARGS (fourcc));
@@ -988,8 +1000,24 @@ gst_v4l2_get_caps_info (GstV4l2Src * v4l2src, GstCaps * caps,
     fourcc = V4L2_PIX_FMT_DV;
   } else if (strcmp (mimetype, "image/jpeg") == 0) {
     fourcc = V4L2_PIX_FMT_JPEG;
+#ifdef V4L2_PIX_FMT_SBGGR8
   } else if (strcmp (mimetype, "video/x-raw-bayer") == 0) {
     fourcc = V4L2_PIX_FMT_SBGGR8;
+#endif
+#ifdef V4L2_PIX_FMT_SN9C10X
+  } else if (strcmp (mimetype, "video/x-sonix") == 0) {
+    fourcc = V4L2_PIX_FMT_SN9C10X;
+#endif
+#ifdef V4L2_PIX_FMT_PWC1
+  } else if (strcmp (mimetype, "video/x-pwc1") == 0) {
+    fourcc = V4L2_PIX_FMT_PWC1;
+#endif
+#ifdef V4L2_PIX_FMT_PWC2
+  } else if (strcmp (mimetype, "video/x-pwc2") == 0) {
+    fourcc = V4L2_PIX_FMT_PWC2;
+#endif
+  } else if (strcmp (mimetype, "video/x-raw-gray") == 0) {
+    fourcc = V4L2_PIX_FMT_GREY;
   }
 
   if (fourcc == 0)
@@ -1071,20 +1099,25 @@ gst_v4l2src_query (GstBaseSrc * bsrc, GstQuery * query)
       GstClockTime min_latency, max_latency;
 
       /* device must be open */
-      if (!GST_V4L2_IS_OPEN (src->v4l2object))
+      if (!GST_V4L2_IS_OPEN (src->v4l2object)) {
+        GST_WARNING_OBJECT (src,
+            "Can't give latency since device isn't open !");
         goto done;
+      }
 
       /* we must have a framerate */
-      if (src->fps_n <= 0 || src->fps_d <= 0)
+      if (src->fps_n <= 0 || src->fps_d <= 0) {
+        GST_WARNING_OBJECT (src,
+            "Can't give latency since framerate isn't fixated !");
         goto done;
+      }
 
       /* min latency is the time to capture one frame */
       min_latency =
           gst_util_uint64_scale_int (GST_SECOND, src->fps_d, src->fps_n);
 
       /* max latency is total duration of the frame buffer */
-      /* FIXME: what to use here? */
-      max_latency = 1 * min_latency;
+      max_latency = src->num_buffers * min_latency;
 
       GST_DEBUG_OBJECT (bsrc,
           "report latency min %" GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
@@ -1162,7 +1195,7 @@ gst_v4l2src_get_read (GstV4l2Src * v4l2src, GstBuffer ** buf)
 
   do {
     amount =
-        read (v4l2src->v4l2object->video_fd, GST_BUFFER_DATA (*buf),
+        v4l2_read (v4l2src->v4l2object->video_fd, GST_BUFFER_DATA (*buf),
         buffersize);
     if (amount == buffersize) {
       break;
@@ -1222,8 +1255,8 @@ gst_v4l2src_get_read (GstV4l2Src * v4l2src, GstBuffer ** buf)
   /* ERRORS */
 read_error:
   {
-    GST_ELEMENT_ERROR (v4l2src, RESOURCE, SYNC,
-        (_("Error read()ing %d bytes on device '%s'."),
+    GST_ELEMENT_ERROR (v4l2src, RESOURCE, READ,
+        (_("Error reading %d bytes from device '%s'."),
             buffersize, v4l2src->v4l2object->videodev), GST_ERROR_SYSTEM);
     gst_buffer_unref (*buf);
     return GST_FLOW_ERROR;
@@ -1272,7 +1305,7 @@ done:
 size_error:
   {
     GST_ELEMENT_ERROR (v4l2src, RESOURCE, READ,
-        (_("Error read()ing %d bytes on device '%s'."),
+        (_("Error reading %d bytes on device '%s'."),
             v4l2src->frame_byte_size, v4l2src->v4l2object->videodev), (NULL));
     return GST_FLOW_ERROR;
   }
