@@ -67,6 +67,7 @@ static GstBufferClass *v4l2buffer_parent_class = NULL;
 static gboolean
 gst_v4l2src_get_nearest_size (GstV4l2Src * v4l2src, guint32 pixelformat,
     gint * width, gint * height);
+static void gst_v4l2_buffer_pool_destroy (GstV4l2BufferPool * pool);
 
 static void
 gst_v4l2_buffer_finalize (GstV4l2Buffer * buffer)
@@ -87,7 +88,7 @@ gst_v4l2_buffer_finalize (GstV4l2Buffer * buffer)
     pool->num_live_buffers--;
 
   if (pool->running) {
-    if (ioctl (pool->video_fd, VIDIOC_QBUF, &buffer->vbuffer) < 0) {
+    if (v4l2_ioctl (pool->video_fd, VIDIOC_QBUF, &buffer->vbuffer) < 0) {
       GST_WARNING ("could not requeue buffer %p %d", buffer, index);
     } else {
       /* FIXME: check that the caps didn't change */
@@ -105,19 +106,13 @@ gst_v4l2_buffer_finalize (GstV4l2Buffer * buffer)
   if (!resuscitated) {
     GST_LOG ("buffer %p not recovered, unmapping", buffer);
     gst_mini_object_unref (GST_MINI_OBJECT (pool));
-    munmap ((void *) GST_BUFFER_DATA (buffer), buffer->vbuffer.length);
+    v4l2_munmap ((void *) GST_BUFFER_DATA (buffer), buffer->vbuffer.length);
 
     GST_MINI_OBJECT_CLASS (v4l2buffer_parent_class)->finalize (GST_MINI_OBJECT
         (buffer));
   }
 }
 
-static void
-gst_v4l2_buffer_init (GstV4l2Buffer * xvimage, gpointer g_class)
-{
-  /* NOP */
-}
-
 static void
 gst_v4l2_buffer_class_init (gpointer g_class, gpointer class_data)
 {
@@ -144,7 +139,7 @@ gst_v4l2_buffer_get_type (void)
       NULL,
       sizeof (GstV4l2Buffer),
       0,
-      (GInstanceInitFunc) gst_v4l2_buffer_init,
+      NULL,
       NULL
     };
     _gst_v4l2_buffer_type = g_type_register_static (GST_TYPE_BUFFER,
@@ -163,15 +158,14 @@ gst_v4l2_buffer_new (GstV4l2BufferPool * pool, guint index, GstCaps * caps)
 
   GST_LOG ("creating buffer %u, %p in pool %p", index, ret, pool);
 
-  ret->pool = pool;
-  gst_mini_object_ref (GST_MINI_OBJECT (pool));
-  memset (&ret->vbuffer, 0x00, sizeof (ret->vbuffer));
+  ret->pool =
+      (GstV4l2BufferPool *) gst_mini_object_ref (GST_MINI_OBJECT (pool));
 
   ret->vbuffer.index = index;
   ret->vbuffer.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
   ret->vbuffer.memory = V4L2_MEMORY_MMAP;
 
-  if (ioctl (pool->video_fd, VIDIOC_QUERYBUF, &ret->vbuffer) < 0)
+  if (v4l2_ioctl (pool->video_fd, VIDIOC_QUERYBUF, &ret->vbuffer) < 0)
     goto querybuf_failed;
 
   GST_LOG ("  index:     %u", ret->vbuffer.index);
@@ -185,7 +179,7 @@ gst_v4l2_buffer_new (GstV4l2BufferPool * pool, guint index, GstCaps * caps)
   GST_LOG ("  length:    %u", ret->vbuffer.length);
   GST_LOG ("  input:     %u", ret->vbuffer.input);
 
-  data = (guint8 *) mmap (0, ret->vbuffer.length,
+  data = (guint8 *) v4l2_mmap (0, ret->vbuffer.length,
       PROT_READ | PROT_WRITE, MAP_SHARED, pool->video_fd,
       ret->vbuffer.m.offset);
 
@@ -235,11 +229,13 @@ gst_v4l2_buffer_pool_finalize (GstV4l2BufferPool * pool)
   pool->lock = NULL;
 
   if (pool->video_fd >= 0)
-    close (pool->video_fd);
+    v4l2_close (pool->video_fd);
 
-  if (pool->buffers)
+  if (pool->buffers) {
     g_free (pool->buffers);
-  pool->buffers = NULL;
+    pool->buffers = NULL;
+  }
+
   GST_MINI_OBJECT_CLASS (buffer_pool_parent_class)->finalize (GST_MINI_OBJECT
       (pool));
 }
@@ -296,7 +292,7 @@ gst_v4l2_buffer_pool_new (GstV4l2Src * v4l2src, gint fd, gint num_buffers,
 
   pool = (GstV4l2BufferPool *) gst_mini_object_new (GST_TYPE_V4L2_BUFFER_POOL);
 
-  pool->video_fd = dup (fd);
+  pool->video_fd = v4l2_dup (fd);
   if (pool->video_fd < 0)
     goto dup_failed;
 
@@ -326,7 +322,7 @@ buffer_new_failed:
   {
     gint errnosave = errno;
 
-    gst_mini_object_unref (GST_MINI_OBJECT (pool));
+    gst_v4l2_buffer_pool_destroy (pool);
 
     errno = errnosave;
 
@@ -348,7 +344,7 @@ gst_v4l2_buffer_pool_activate (GstV4l2BufferPool * pool, GstV4l2Src * v4l2src)
 
     GST_LOG ("enqueue pool buffer %d", n);
 
-    if (ioctl (pool->video_fd, VIDIOC_QBUF, buf) < 0)
+    if (v4l2_ioctl (pool->video_fd, VIDIOC_QBUF, buf) < 0)
       goto queue_failed;
   }
   pool->running = TRUE;
@@ -379,10 +375,14 @@ gst_v4l2_buffer_pool_destroy (GstV4l2BufferPool * pool)
   pool->running = FALSE;
   g_mutex_unlock (pool->lock);
 
+  GST_DEBUG ("destroy pool");
+
   /* after this point, no more buffers will be queued or dequeued; no buffer
    * from pool->buffers that is NULL will be set to a buffer, and no buffer that
    * is not NULL will be pushed out. */
 
+  /* miniobjects have no dispose, so they can't break ref-cycles, as buffers ref
+   * the pool, we need to unref the buffer to properly finalize te pool */
   for (n = 0; n < pool->buffer_count; n++) {
     GstBuffer *buf;
 
@@ -406,7 +406,9 @@ gst_v4l2_buffer_pool_destroy (GstV4l2BufferPool * pool)
 #define YUV_ODD_BASE_RANK   50
 #define RGB_ODD_BASE_RANK   25
 #define BAYER_BASE_RANK     15
+#define S910_BASE_RANK      10
 #define GREY_BASE_RANK       5
+#define PWC_BASE_RANK        1
 
 static gint
 gst_v4l2src_format_get_rank (guint32 fourcc)
@@ -467,8 +469,24 @@ gst_v4l2src_format_get_rank (guint32 fourcc)
     case V4L2_PIX_FMT_WNVA:    /* Winnov hw compres */
       return 0;
 
+#ifdef V4L2_PIX_FMT_SBGGR8
     case V4L2_PIX_FMT_SBGGR8:
       return BAYER_BASE_RANK;
+#endif
+
+#ifdef V4L2_PIX_FMT_SN9C10X
+    case V4L2_PIX_FMT_SN9C10X:
+      return S910_BASE_RANK;
+#endif
+
+#ifdef V4L2_PIX_FMT_PWC1
+    case V4L2_PIX_FMT_PWC1:
+      return PWC_BASE_RANK;
+#endif
+#ifdef V4L2_PIX_FMT_PWC2
+    case V4L2_PIX_FMT_PWC2:
+      return PWC_BASE_RANK;
+#endif
 
     default:
       break;
@@ -509,7 +527,7 @@ gst_v4l2src_fill_format_list (GstV4l2Src * v4l2src)
     format->index = n;
     format->type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 
-    if (ioctl (v4l2src->v4l2object->video_fd, VIDIOC_ENUM_FMT, format) < 0) {
+    if (v4l2_ioctl (v4l2src->v4l2object->video_fd, VIDIOC_ENUM_FMT, format) < 0) {
       if (errno == EINVAL) {
         g_free (format);
         break;                  /* end of enumeration */
@@ -585,7 +603,7 @@ gst_v4l2src_probe_caps_for_format_and_size (GstV4l2Src * v4l2src,
 
   /* keep in mind that v4l2 gives us frame intervals (durations); we invert the
    * fraction to get framerate */
-  if (ioctl (fd, VIDIOC_ENUM_FRAMEINTERVALS, &ival) < 0)
+  if (v4l2_ioctl (fd, VIDIOC_ENUM_FRAMEINTERVALS, &ival) < 0)
     goto enum_frameintervals_failed;
 
   if (ival.type == V4L2_FRMIVAL_TYPE_DISCRETE) {
@@ -611,7 +629,7 @@ gst_v4l2src_probe_caps_for_format_and_size (GstV4l2Src * v4l2src,
       gst_value_list_append_value (&rates, &rate);
 
       ival.index++;
-    } while (ioctl (fd, VIDIOC_ENUM_FRAMEINTERVALS, &ival) >= 0);
+    } while (v4l2_ioctl (fd, VIDIOC_ENUM_FRAMEINTERVALS, &ival) >= 0);
   } else if (ival.type == V4L2_FRMIVAL_TYPE_STEPWISE) {
     GValue min = { 0, };
     GValue step = { 0, };
@@ -753,6 +771,20 @@ unknown_type:
 }
 #endif /* defined VIDIOC_ENUM_FRAMEINTERVALS */
 
+static gint
+sort_by_frame_size (GstStructure * s1, GstStructure * s2)
+{
+  int w1, h1, w2, h2;
+
+  gst_structure_get_int (s1, "width", &w1);
+  gst_structure_get_int (s1, "height", &h1);
+  gst_structure_get_int (s2, "width", &w2);
+  gst_structure_get_int (s2, "height", &h2);
+
+  /* I think it's safe to assume that this won't overflow for a while */
+  return ((w2 * h2) - (w1 * h1));
+}
+
 GstCaps *
 gst_v4l2src_probe_caps_for_format (GstV4l2Src * v4l2src, guint32 pixelformat,
     const GstStructure * template)
@@ -770,25 +802,37 @@ gst_v4l2src_probe_caps_for_format (GstV4l2Src * v4l2src, guint32 pixelformat,
   size.index = 0;
   size.pixel_format = pixelformat;
 
-  if (ioctl (fd, VIDIOC_ENUM_FRAMESIZES, &size) < 0)
+  GST_DEBUG_OBJECT (v4l2src, "Enumerating frame sizes");
+
+  if (v4l2_ioctl (fd, VIDIOC_ENUM_FRAMESIZES, &size) < 0)
     goto enum_framesizes_failed;
 
   if (size.type == V4L2_FRMSIZE_TYPE_DISCRETE) {
     do {
+      GST_LOG_OBJECT (v4l2src, "got discrete frame size %dx%d",
+          size.discrete.width, size.discrete.height);
+
       w = MIN (size.discrete.width, G_MAXINT);
       h = MIN (size.discrete.height, G_MAXINT);
 
       tmp = gst_v4l2src_probe_caps_for_format_and_size (v4l2src, pixelformat,
           w, h, template);
 
-      /* we get low res to high res, but want high res to low res in caps, so
-       * prepend structs to results list, we'll reverse the order later then */
       if (tmp)
         results = g_list_prepend (results, tmp);
 
       size.index++;
-    } while (ioctl (fd, VIDIOC_ENUM_FRAMESIZES, &size) >= 0);
+    } while (v4l2_ioctl (fd, VIDIOC_ENUM_FRAMESIZES, &size) >= 0);
+    GST_DEBUG_OBJECT (v4l2src, "done iterating discrete frame sizes");
   } else if (size.type == V4L2_FRMSIZE_TYPE_STEPWISE) {
+    GST_DEBUG_OBJECT (v4l2src, "we have stepwise frame sizes:");
+    GST_DEBUG_OBJECT (v4l2src, "min width:   %d", size.stepwise.min_width);
+    GST_DEBUG_OBJECT (v4l2src, "min height:  %d", size.stepwise.min_height);
+    GST_DEBUG_OBJECT (v4l2src, "max width:   %d", size.stepwise.max_width);
+    GST_DEBUG_OBJECT (v4l2src, "min height:  %d", size.stepwise.max_height);
+    GST_DEBUG_OBJECT (v4l2src, "step width:  %d", size.stepwise.step_width);
+    GST_DEBUG_OBJECT (v4l2src, "step height: %d", size.stepwise.step_height);
+
     for (w = size.stepwise.min_width, h = size.stepwise.min_height;
         w < size.stepwise.max_width && h < size.stepwise.max_height;
         w += size.stepwise.step_width, h += size.stepwise.step_height) {
@@ -798,14 +842,19 @@ gst_v4l2src_probe_caps_for_format (GstV4l2Src * v4l2src, guint32 pixelformat,
       tmp = gst_v4l2src_probe_caps_for_format_and_size (v4l2src, pixelformat,
           w, h, template);
 
-      /* we get low res to high res, but want high res to low res in caps, so
-       * prepend structs to results list, we'll reverse the order later then */
       if (tmp)
         results = g_list_prepend (results, tmp);
     }
+    GST_DEBUG_OBJECT (v4l2src, "done iterating stepwise frame sizes");
   } else if (size.type == V4L2_FRMSIZE_TYPE_CONTINUOUS) {
     guint32 maxw, maxh;
 
+    GST_DEBUG_OBJECT (v4l2src, "we have continuous frame sizes:");
+    GST_DEBUG_OBJECT (v4l2src, "min width:   %d", size.stepwise.min_width);
+    GST_DEBUG_OBJECT (v4l2src, "min height:  %d", size.stepwise.min_height);
+    GST_DEBUG_OBJECT (v4l2src, "max width:   %d", size.stepwise.max_width);
+    GST_DEBUG_OBJECT (v4l2src, "min height:  %d", size.stepwise.max_height);
+
     w = MAX (size.stepwise.min_width, 1);
     h = MAX (size.stepwise.min_height, 1);
     maxw = MIN (size.stepwise.max_width, G_MAXINT);
@@ -825,12 +874,12 @@ gst_v4l2src_probe_caps_for_format (GstV4l2Src * v4l2src, guint32 pixelformat,
     goto unknown_type;
   }
 
-  /* we use an intermediary list to store the results of the probing because
-   * we probe from lowest resolution to highest resolution, but want the caps
-   * to contain the results in reverse order starting with the highest
-   * resolution, as order in caps matters for things like fixation.  However,
-   * there's no gst_caps_prepend_structure(), so we use the list as helper to
-   * reverse the order */
+  /* we use an intermediary list to store and then sort the results of the
+   * probing because we can't make any assumptions about the order in which
+   * the driver will give us the sizes, but we want the final caps to contain
+   * the results starting with the highest resolution and having the lowest
+   * resolution last, since order in caps matters for things like fixation. */
+  results = g_list_sort (results, (GCompareFunc) sort_by_frame_size);
   while (results != NULL) {
     gst_caps_append_structure (ret, GST_STRUCTURE (results->data));
     results = g_list_delete_link (results, results);
@@ -913,7 +962,8 @@ default_frame_sizes:
       /* if norm can't be used, copy the template framerate */
       gst_structure_set (tmp,
           "width", GST_TYPE_INT_RANGE, min_w, max_w,
-          "height", GST_TYPE_INT_RANGE, min_h, max_h, NULL);
+          "height", GST_TYPE_INT_RANGE, min_h, max_h,
+          "framerate", GST_TYPE_FRACTION_RANGE, 0, 1, 100, 1, NULL);
     }
     gst_caps_append_structure (ret, tmp);
 
@@ -924,7 +974,7 @@ default_frame_sizes:
 /******************************************************
  * gst_v4l2src_grab_frame ():
  *   grab a frame for capturing
- * return value: The captured frame number or -1 on error.
+ * return value: GST_FLOW_OK or GST_FLOW_ERROR
  ******************************************************/
 GstFlowReturn
 gst_v4l2src_grab_frame (GstV4l2Src * v4l2src, GstBuffer ** buf)
@@ -940,7 +990,7 @@ gst_v4l2src_grab_frame (GstV4l2Src * v4l2src, GstBuffer ** buf)
   buffer.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
   buffer.memory = V4L2_MEMORY_MMAP;
 
-  while (ioctl (v4l2src->v4l2object->video_fd, VIDIOC_DQBUF, &buffer) < 0) {
+  while (v4l2_ioctl (v4l2src->v4l2object->video_fd, VIDIOC_DQBUF, &buffer) < 0) {
 
     GST_WARNING_OBJECT (v4l2src,
         "problem grabbing frame %d (ix=%d), trials=%d, pool-ct=%d, buf.flags=%d",
@@ -1131,19 +1181,10 @@ qbuf_failed:
 */
 }
 
-static gboolean
-fractions_are_equal (gint num1, gint den1, gint num2, gint den2)
-{
-  GValue fraction1 = { 0, }, fraction2 = {
-  0,};
-
-  g_value_init (&fraction1, GST_TYPE_FRACTION);
-  g_value_init (&fraction2, GST_TYPE_FRACTION);
-  gst_value_set_fraction (&fraction1, num1, den1);
-  gst_value_set_fraction (&fraction2, num2, den2);
-  /* we know we don't have to unset the values in this case */
-  return (gst_value_compare (&fraction1, &fraction2) == GST_VALUE_EQUAL);
-}
+/* Note about fraction simplification
+ * n1/d1 == n2/d2  is also written as  n1 == ( n2 * d1 ) / d2
+ */
+#define fractions_are_equal(n1,d1,n2,d2) ((n1) == gst_util_uint64_scale_int((n2), (d1), (d2)))
 
 /******************************************************
  * gst_v4l2src_set_capture():
@@ -1167,7 +1208,7 @@ gst_v4l2src_set_capture (GstV4l2Src * v4l2src, guint32 pixelformat,
   memset (&format, 0x00, sizeof (struct v4l2_format));
   format.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 
-  if (ioctl (fd, VIDIOC_G_FMT, &format) < 0)
+  if (v4l2_ioctl (fd, VIDIOC_G_FMT, &format) < 0)
     goto get_fmt_failed;
 
   format.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
@@ -1179,7 +1220,7 @@ gst_v4l2src_set_capture (GstV4l2Src * v4l2src, guint32 pixelformat,
    *  combined, there are other modes for requesting fields individually) */
   format.fmt.pix.field = V4L2_FIELD_INTERLACED;
 
-  if (ioctl (fd, VIDIOC_S_FMT, &format) < 0) {
+  if (v4l2_ioctl (fd, VIDIOC_S_FMT, &format) < 0) {
     if (errno != EINVAL)
       goto set_fmt_failed;
 
@@ -1188,7 +1229,7 @@ gst_v4l2src_set_capture (GstV4l2Src * v4l2src, guint32 pixelformat,
     format.fmt.pix.height = height;
     format.fmt.pix.pixelformat = pixelformat;
     format.fmt.pix.field = V4L2_FIELD_NONE;
-    if (ioctl (fd, VIDIOC_S_FMT, &format) < 0)
+    if (v4l2_ioctl (fd, VIDIOC_S_FMT, &format) < 0)
       goto set_fmt_failed;
   }
 
@@ -1203,7 +1244,7 @@ gst_v4l2src_set_capture (GstV4l2Src * v4l2src, guint32 pixelformat,
 
   memset (&stream, 0x00, sizeof (struct v4l2_streamparm));
   stream.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-  if (ioctl (fd, VIDIOC_G_PARM, &stream) < 0) {
+  if (v4l2_ioctl (fd, VIDIOC_G_PARM, &stream) < 0) {
     GST_ELEMENT_WARNING (v4l2src, RESOURCE, SETTINGS,
         (_("Could not get parameters on device '%s'"),
             v4l2src->v4l2object->videodev), GST_ERROR_SYSTEM);
@@ -1213,7 +1254,9 @@ gst_v4l2src_set_capture (GstV4l2Src * v4l2src, guint32 pixelformat,
   /* Note: V4L2 provides the frame interval, we have the frame rate */
   if (fractions_are_equal (stream.parm.capture.timeperframe.numerator,
           stream.parm.capture.timeperframe.denominator, fps_d, fps_n)) {
-    GST_LOG_OBJECT (v4l2src, "Desired framerate already set, nothing to do");
+    GST_LOG_OBJECT (v4l2src, "Desired framerate already set");
+    v4l2src->fps_n = fps_n;
+    v4l2src->fps_d = fps_d;
     goto done;
   }
 
@@ -1231,7 +1274,7 @@ gst_v4l2src_set_capture (GstV4l2Src * v4l2src, guint32 pixelformat,
   stream.parm.capture.timeperframe.denominator = fps_n;
 
   /* some cheap USB cam's won't accept any change */
-  if (ioctl (fd, VIDIOC_S_PARM, &stream) < 0) {
+  if (v4l2_ioctl (fd, VIDIOC_S_PARM, &stream) < 0) {
     GST_ELEMENT_WARNING (v4l2src, RESOURCE, SETTINGS,
         (_("Video input device did not accept new frame rate setting.")),
         GST_ERROR_SYSTEM);
@@ -1313,7 +1356,7 @@ gst_v4l2src_capture_init (GstV4l2Src * v4l2src, GstCaps * caps)
     breq.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
     breq.memory = V4L2_MEMORY_MMAP;
 
-    if (ioctl (fd, VIDIOC_REQBUFS, &breq) < 0)
+    if (v4l2_ioctl (fd, VIDIOC_REQBUFS, &breq) < 0)
       goto reqbufs_failed;
 
     GST_LOG_OBJECT (v4l2src, " count:  %u", breq.count);
@@ -1408,7 +1451,7 @@ gst_v4l2src_capture_start (GstV4l2Src * v4l2src)
     if (!gst_v4l2_buffer_pool_activate (v4l2src->pool, v4l2src))
       goto pool_activate_failed;
 
-    if (ioctl (fd, VIDIOC_STREAMON, &type) < 0)
+    if (v4l2_ioctl (fd, VIDIOC_STREAMON, &type) < 0)
       goto streamon_failed;
   }
 
@@ -1453,7 +1496,7 @@ gst_v4l2src_capture_stop (GstV4l2Src * v4l2src)
   if (v4l2src->use_mmap) {
     /* we actually need to sync on all queued buffers but not
      * on the non-queued ones */
-    if (ioctl (v4l2src->v4l2object->video_fd, VIDIOC_STREAMOFF, &type) < 0)
+    if (v4l2_ioctl (v4l2src->v4l2object->video_fd, VIDIOC_STREAMOFF, &type) < 0)
       goto streamoff_failed;
   }
 
@@ -1529,14 +1572,14 @@ gst_v4l2src_get_nearest_size (GstV4l2Src * v4l2src, guint32 pixelformat,
   fmt.fmt.pix.pixelformat = pixelformat;
   fmt.fmt.pix.field = V4L2_FIELD_INTERLACED;
 
-  r = ioctl (fd, VIDIOC_TRY_FMT, &fmt);
+  r = v4l2_ioctl (fd, VIDIOC_TRY_FMT, &fmt);
   if (r < 0 && errno == EINVAL) {
     /* try again with progressive video */
     fmt.fmt.pix.width = *width;
     fmt.fmt.pix.height = *height;
     fmt.fmt.pix.pixelformat = pixelformat;
     fmt.fmt.pix.field = V4L2_FIELD_NONE;
-    r = ioctl (fd, VIDIOC_TRY_FMT, &fmt);
+    r = v4l2_ioctl (fd, VIDIOC_TRY_FMT, &fmt);
   }
 
   if (r < 0) {
@@ -1556,14 +1599,14 @@ gst_v4l2src_get_nearest_size (GstV4l2Src * v4l2src, guint32 pixelformat,
     fmt.fmt.pix.width = *width;
     fmt.fmt.pix.height = *height;
 
-    r = ioctl (fd, VIDIOC_S_FMT, &fmt);
+    r = v4l2_ioctl (fd, VIDIOC_S_FMT, &fmt);
     if (r < 0 && errno == EINVAL) {
       /* try again with progressive video */
       fmt.fmt.pix.width = *width;
       fmt.fmt.pix.height = *height;
       fmt.fmt.pix.pixelformat = pixelformat;
       fmt.fmt.pix.field = V4L2_FIELD_NONE;
-      r = ioctl (fd, VIDIOC_S_FMT, &fmt);
+      r = v4l2_ioctl (fd, VIDIOC_S_FMT, &fmt);
     }
 
     if (r < 0)
